/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
'use strict'

// handle setupevents as quickly as possible
/*
const setupEvents = require('./setupEvents')
if (setupEvents.handleSquirrelEvent()) {
  // squirrel event handled and app will exit in 1000ms, so don't do anything else
  return
}
*/

const electron = require('electron')
const { app, Menu } = require('electron')

const BrowserWindow = electron.BrowserWindow
const dialog = require('electron').dialog
const ipc = electron.ipcMain
const fs = require('fs')
const path = require('path')
const clipboard = require('electron').clipboard
const electronLocalshortcut = require('electron-localshortcut')
const Store = require('./store')
const nativeImage = require('electron').nativeImage
const textFr = require('./text/fr')
const textEn = require('./text/en')
const textEs = require('./text/es')

// A metre à la main à true pour déboguer
// const isDebug = true
const isDebug = process.argv.includes('--debug')
// on doit charger un chemin absolu
let url = `file://${__dirname}/index.html`
// si on passe du --debug=http://localhost:8081/index.debug.html on ira prendre cette url de départ
if (isDebug) {
  const chunks = /--debug=([^ ]+)/.exec(process.argv)
  const debugUrl = chunks && chunks[1]
  if (debugUrl) url = debugUrl
}

let language // Sur Windows getLocale doit être appelé après ready

let filePath = '' // Contiendra le chemin d'accès au fichier lors de la sauvegarde
let newFilePath // Contient le chemin d'accès à une figure à ouvrir en attendant qu'on soit sur que la figure est valide
let mainWindow
let bmayClose = true // Boolean contenant true si la fenêtre a le droit de se fermer et false sinon

const store = new Store({
  // Notre fichier de configuration sera nommé user-preferences
  configName: 'user-preferences',
  defaults: {
    // 900x600 est la taille par défaut de la fenêtre
    // Si l'utilisateur change la taille de la fenêtre celle-ci sera mémorisée et utilisée
    // lors du prochain lancement
    windowSize: {
      width: 900,
      height: 650
    },
    level: '3',
    displayMeasures: 'true',
    pointsAuto: 'true',
    dysmode: 'false',
    isMaximized: false,
    startFig: 'frameGrid' // Autres valeurs possibles : frameDotted, unity
  }
})

function createWindow (startFigure) {
  const { width, height } = store.get('windowSize')
  const level = store.get('level')
  const dysmode = store.get('dysmode')
  const displayMeasures = store.get('displayMeasures')
  const pointsAuto = dysmode ? false : store.get('pointsAuto')
  const startFig = arguments.length === 0 ? store.get('startFig') : startFigure

  electron.Menu.setApplicationMenu(null) // Avant instance de mainWindow
  // On passe les options
  mainWindow = new BrowserWindow({
    width: width,
    height: height,
    title: 'MathGraph32 JavaScript',
    // icon: 'logoMathGraph.png',
    icon: path.resolve(__dirname, 'logoMathGraph.png'),
    center: true,
    show: false, // Pour qu'on puisse appeler resize et bien positionner la barre d'outils de droite
    webPreferences: {
      nativeWindowOpen: true,
      nodeIntegration: true
    }
  })

  // createMenu(); // Abandonné car maintenant on peut cliquer sur les icônes du logiciel
  // mainWindow.setMenu(null)
  if (isDebug) mainWindow.webContents.openDevTools() // Ligne qui doit être présente pour debug

  mainWindow.level = level
  mainWindow.displayMeasures = displayMeasures
  mainWindow.pointsAuto = pointsAuto
  mainWindow.dysmode = dysmode
  // mainWindow.startFig = startFig
  // on définit une taille minimale pour notre fenêtre
  mainWindow.setMinimumSize(800, 520)
  mainWindow.loadURL(url)

  const size = mainWindow.getContentSize()
  const w = size[0] - 16
  const h = size[1] - 20

  // ////////////////////////////////////////////////////////////////
  const commands = {
    // Ctrl + C pour copier la figure dans le presse-papier
    'copyFig()': 'CommandOrControl+C',
    // Ctrl + S pour enregistrer la figure
    'saveForIcon()': 'CommandOrControl+S',
    // Ctrl + O pour ouvrir une figure
    'openFile()': 'CommandOrControl+O',
    // F1 pour lancer l'aide dans le navigateur
    'help()': 'F1',
    // F2 pour l'outil précédent
    'activeOutilPrec()': 'F2',
    // F10 pour réafficher la dernière indication fugitive
    'lastIndication()': 'F10'
  }
  Object.entries(commands).forEach(([command, shortcut]) => {
    electronLocalshortcut.register(mainWindow, shortcut, () => {
      mainWindow.webContents.executeJavaScript(command)
    })
  })

  // Traitement des raccourcis clavier pour les outils à activer
  const tools = {
    ModifObjNum: 'F3',
    Nommer: 'F4',
    CaptNom: 'F5',
    Gomme: 'F6',
    Rideau: 'F7',
    ExecutionMacro: 'F8',
    Protocole: 'F9',
    PtParCoord: 'CommandOrControl+Insert',
    Sup: 'CommandOrControl+Delete',
    Capt: 'Escape',
    Seg: 'CommandOrControl+A',
    CerOA: 'CommandOrControl+B',
    CerOR: 'CommandOrControl+Shift+B',
    DtAB: 'CommandOrControl+D',
    DtParCoef: 'CommandOrControl+Shift+D',
    Calcul: 'CommandOrControl+E',
    CalculComp: 'CommandOrControl+Shift+E',
    Fonc: 'CommandOrControl+F',
    FoncComp: 'CommandOrControl+Shift+F',
    Trans: 'CommandOrControl+G',
    Curseur: 'CommandOrControl+J',
    Polygone: 'CommandOrControl+K',
    Surface: 'CommandOrControl+Shift+K',
    MesLong: 'CommandOrControl+L',
    MesAngNor: 'CommandOrControl+Shift+L',
    MarqueSeg: 'CommandOrControl+M',
    MarqueAng: 'CommandOrControl+Shift+M',
    New: 'CommandOrControl+N',
    PtLib: 'CommandOrControl+P',
    PtLie: 'CommandOrControl+Shift+P',
    Commentaire: 'CommandOrControl+T',
    CommentaireLiePt: 'CommandOrControl+Shift+T',
    AffichageValeur: 'CommandOrControl+U',
    AffichageValeurLiePt: 'CommandOrControl+Shift+U',
    Latex: 'CommandOrControl+V',
    LatexLiePt: 'CommandOrControl+Shift+V',
    Annuler: 'CommandOrControl+Z',
    Refaire: 'CommandOrControl+Shift+Z'
  }
  Object.entries(tools).forEach(([tool, shortcut]) => {
    electronLocalshortcut.register(mainWindow, shortcut, () => {
      mainWindow.webContents.executeJavaScript(`activateTool('${tool}')`)
    })
  })
  /// ///////////////////////////////////////////////////////////////
  // Attention ci-dessous il faut des guillemets et pas de ' sinon plantages au démarrage sur double clic
  // sans doute parce que startfig peut contenir des caractères '
  mainWindow.webContents.executeJavaScript(`start("${language}", ${w}, ${h}, "${level}", ${displayMeasures}, ${pointsAuto}, ${dysmode}, "${startFig}")`)
  // mainWindow.webContents.executeJavaScript('start("' + language + '",' + w + ',' + h + ',"' + level +
  //    '",' + displayMeasures + ',' + pointsAuto + ',' + dysmode + ',"' + startFig + '")')
  if (store.get('isMaximized')) mainWindow.maximize()
  mainWindow.show()

  mainWindow.on('resize', () => {
    setSize()
  })

  mainWindow.on('maximize', () => {
    setTimeout(() => {
      setSize()
    }, 300)
  })

  mainWindow.on('restore', () => {
    setTimeout(() => {
      setSize()
    }, 300)
  })

  mainWindow.on('unmaximize', () => {
    setTimeout(() => {
      setSize()
    }, 300)
  })

  mainWindow.on('enter-full-screen', () => {
    setSize()
  })

  mainWindow.on('leave-full-screen', () => {
    setSize()
  })

  mainWindow.on('closed', () => {
    mainWindow = null
  })

  mainWindow.on('close', (ev) => {
    const { width, height } = mainWindow.getBounds()
    // On enregistre les dimensions de la fenêtre dans les préférences avant de quitter
    store.set('windowSize', { width, height })
    store.set('language', language)
    store.set('level', mainWindow.level)
    store.set('dysmode', mainWindow.dysmode)
    store.set('isMaximized', mainWindow.isMaximized())
    store.set('displayMeasures', mainWindow.displayMeasures)
    store.set('pointsAuto', mainWindow.pointsAuto)
    // store.set('startFig', mainWindow.startFig)

    if (!bmayClose) {
      const buttons = [getStr('Yes'), getStr('No')]
      ev.preventDefault()
      dialog.showMessageBox(mainWindow, {
        type: 'warning',
        buttons: buttons,
        noLink: true,
        message: getStr('AvertDirty')
      }).then(result => {
        if (result.response === 0) {
          bmayClose = true
          mainWindow.close()
        }
      }).catch(err => {
        console.log(err)
      })
    }
  })
}

function setSize () {
  const size = mainWindow.getContentSize()
  const w = size[0] - 16
  const h = size[1] - 20
  mainWindow.webContents.executeJavaScript('resize(' + w + ',' + h + ')')
}

function start () {
  if (store.get('language') !== undefined) language = store.get('language')
  else getLanguage()

  // Check if we are on a MAC
  if (process.platform === 'darwin') {
    // Create our menu entries so that we can use MAC shortcuts
    Menu.setApplicationMenu(Menu.buildFromTemplate([
      {
        label: 'File',
        submenu: [
          { label: getStr('quit'), role: 'quit' }
        ]
      },
      {
        label: 'Edit',
        submenu: [
          { label: getStr('undo'), role: 'undo' },
          { label: getStr('redo'), role: 'redo' },
          { type: 'separator' },
          { label: getStr('cut'), role: 'cut' },
          { label: getStr('copy'), role: 'copy' },
          { label: getStr('paste'), role: 'paste' },
          { label: getStr('delete'), role: 'delete' },
          { label: getStr('selectall'), role: 'selectall' }
        ]
      }
    ]))
  }

  var args = process.argv
  // On recherche si on est en train d'ouvrir une figure par association de fichier (double clic)
  let found = false
  for (let i = 0; (i < args.length) && !found; i++) {
    const ch = args[i]
    if (ch.endsWith('mgj')) {
      found = true
      fs.readFile(ch, 'latin1', (err, data) => {
        if (err) {
          console.log(getStr('ErrOpen') + err.message)
          return
        }
        newFilePath = ch
        let startFigure = JSON.stringify(data)
        const len = startFigure.length
        startFigure = startFigure.substring(1, len - 1) // Por éliminer les "" de début et de find de la chaîne
        createWindow(startFigure)
      })
    }
  }
  if (!found) createWindow()
}

/**
 * Le message suivant est reçu après qu'on a choisi le menu enregistrer
 * et c'est là que l'enregistrement réel se fait
 */
ipc.on('save', (evt, ba) => {
  fs.writeFile(filePath, ba, (err) => {
    if (err) {
      console.log(getStr('ErrSave') + err.message)
    }
  })
  bmayClose = true
})

ipc.on('savepng', (evt, data) => {
  const parentWindow = (process.platform === 'darwin') ? null : BrowserWindow.getFocusedWindow()
  dialog.showSaveDialog(parentWindow, {
    title: getStr('ExpIm'),
    filters: [
      { name: getStr('PNGFiles'), extensions: ['png', 'PNG'] },
      { name: getStr('AllFiles'), extensions: ['*'] }
    ]
  }).then(result => {
    if (!result.canceled) {
      fs.writeFile(result.filePath, data, { encoding: 'base64' }, (err) => {
        if (err) {
          console.log(getStr('ErrSave') + err.message)
        }
      })
    }
  }).catch(err => {
    console.log(err)
  })
})

ipc.on('savejpg', (evt, data) => {
  const parentWindow = (process.platform === 'darwin') ? null : BrowserWindow.getFocusedWindow()
  dialog.showSaveDialog(parentWindow, {
    title: getStr('ExpIm'),
    filters: [
      { name: getStr('JPGFiles'), extensions: ['jpg', 'JPG', 'jpeg', 'JPEG'] },
      { name: getStr('AllFiles'), extensions: ['*'] }
    ]
  }).then(result => {
    if (!result.canceled) {
      fs.writeFile(result.filePath, data, { encoding: 'base64' }, (err) => {
        if (err) {
          console.log(getStr('ErrSave') + err.message)
        }
      })
    }
  }).catch(err => {
    console.log(err)
  })
})

ipc.on('savesvg', (evt, data) => {
  const parentWindow = (process.platform === 'darwin') ? null : BrowserWindow.getFocusedWindow()
  dialog.showSaveDialog(parentWindow, {
    title: getStr('ExpIm'),
    filters: [
      { name: getStr('SVGFiles'), extensions: ['svg', 'SVG'] },
      { name: getStr('AllFiles'), extensions: ['*'] }
    ]
  }).then(result => {
    if (!result.canceled) {
      fs.writeFile(result.filePath, data, (err) => {
        if (err) {
          console.log(getStr('ErrSave') + err.message)
        }
      })
    }
  }).catch(err => {
    console.log(err)
  })
})

ipc.on('saveproto', (evt, data, title) => {
  const parentWindow = (process.platform === 'darwin') ? null : BrowserWindow.getFocusedWindow()
  dialog.showSaveDialog(parentWindow, {
    title: getStr('SaveProto' + title),
    filters: [
      { name: getStr('MGCFiles'), extensions: ['mgc', 'MGC'] },
      { name: getStr('AllFiles'), extensions: ['*'] }
    ]
  }).then(result => {
    if (!result.canceled) {
      fs.writeFile(result.filePath, data, (err) => {
        if (err) {
          console.log(getStr('ErrSave') + err.message)
        }
      })
    }
  }).catch(err => {
    console.log(err)
  })
})

ipc.on('savehtml', (evt, data) => {
  const parentWindow = (process.platform === 'darwin') ? null : BrowserWindow.getFocusedWindow()
  dialog.showSaveDialog(parentWindow, {
    title: getStr('ExpHTML'),
    filters: [
      { name: getStr('HTMLFiles'), extensions: ['html', 'htm'] },
      { name: getStr('AllFiles'), extensions: ['*'] }
    ]
  }).then(result => {
    if (!result.canceled) {
      fs.writeFile(result.filePath, data, (err) => {
        if (err) {
          console.log(getStr('ErrSave') + err.message)
        }
      })
    }
  }).catch(err => {
    console.log(err)
  })
})

ipc.on('saveForIcon', () => {
  if (filePath === '') saveAs()
  else {
    const buttons = [getStr('Save'), getStr('SaveAs'), getStr('Discard')]
    dialog.showMessageBox(mainWindow, {
      type: 'none',
      buttons: buttons,
      message: getStr('Choice')
    }).then(result => {
      switch (result.response) {
        case 0 :
          save()
          break
        case 1 :
          saveAs()
      }
    }).catch(err => {
      console.log(err)
    })
  }
})

// Ce message est envoyé par index.html quand l'utilisateur a cliqué sur l'icône d'ouverture de fichier
ipc.on('open', () => {
  open()
})

// Ce message est envoyé par index.html quand l'utilisateur a choisi d'incorporer une construction depuis un fichier
ipc.on('addconstfromfile', () => {
  addConst()
})

ipc.on('code', (evt, code) => {
  clipboard.writeText(code)
})

ipc.on('copy', (evt, data) => {
  clipboard.writeImage(nativeImage.createFromBuffer(Buffer.from(data, 'base64')))
})

ipc.on('resetdoc', (evt) => {
  bmayClose = true
  filePath = ''
  mainWindow.setTitle('MathGraph32JS')
})

ipc.on('setdocumentdirty', (evt) => {
  bmayClose = false
})

ipc.on('level', (evt, level) => {
  mainWindow.level = level
})

ipc.on('dysmode', (evt, bdysmode) => {
  mainWindow.dysmode = bdysmode
})

ipc.on('displaymeasures', (evt, bdisplay) => {
  mainWindow.displayMeasures = bdisplay
})

ipc.on('pointsAuto', (evt, bPointsAuto) => {
  mainWindow.pointsAuto = bPointsAuto
})

ipc.on('startFig', (evt, startFig) => {
  store.set('startFig', startFig)
})

ipc.on('setlang', (evt, lang) => {
  language = lang
})

ipc.on('setnewpath', (evt, path) => {
  filePath = path === '' ? newFilePath : path
  var name = filePath.split('\\').pop().split('/').pop()
  mainWindow.setTitle('MathGraph32JS-' + name)
})

app.on('ready', start)

app.on('window-all-closed', () => {
  app.quit()
})

app.on('activate', () => {
  if (mainWindow === null) createWindow()
})

function open () {
  const parentWindow = (process.platform === 'darwin') ? null : BrowserWindow.getFocusedWindow()
  dialog.showOpenDialog(parentWindow, {
    title: getStr('OpenFig'),
    filters: [
      { name: getStr('MtgFiles'), extensions: ['mgj'] },
      { name: getStr('AllFiles'), extensions: ['*'] }
    ]
  }).then(result => {
    if (!result.canceled) {
      fs.readFile(result.filePaths[0], 'latin1', (err, data) => {
        if (err) {
          console.log(getStr('ErrOpen') + err.message)
          return
        }
        newFilePath = result.filePaths[0]
        mainWindow.webContents.executeJavaScript('openFig(' + JSON.stringify(data) + ')')
      })
    }
  }).catch(err => {
    console.log(err)
  })
}

function addConst () {
  const parentWindow = (process.platform === 'darwin') ? null : BrowserWindow.getFocusedWindow()
  dialog.showOpenDialog(parentWindow, {
    title: getStr('AddConst'),
    filters: [
      { name: getStr('MtgFiles'), extensions: ['mgc'] },
      { name: getStr('AllFiles'), extensions: ['*'] }
    ]
  }).then(result => {
    if (!result.canceled) {
      fs.readFile(result.filePaths[0], 'latin1', (err, data) => {
        if (err) {
          console.log(getStr('ErrOpen') + err.message)
          return
        }
        newFilePath = result.filePaths[0]
        mainWindow.webContents.executeJavaScript('addConst(' + JSON.stringify(data) + ')')
      })
    }
  }).catch(err => {
    console.log(err)
  })
}

function saveAs () {
  const parentWindow = (process.platform === 'darwin') ? null : BrowserWindow.getFocusedWindow()
  dialog.showSaveDialog(parentWindow, {
    title: getStr('SaveFig'),
    filters: [
      { name: getStr('MtgFiles'), extensions: ['mgj'] },
      { name: getStr('AllFiles'), extensions: ['*'] }
    ]
  }).then(result => {
    if (!result.canceled) {
      filePath = result.filePath
      const name = filePath.split('\\').pop().split('/').pop()
      mainWindow.setTitle('MathGraph32JS-' + name)
      mainWindow.webContents.executeJavaScript('save()')
    }
  }).catch(err => {
    console.log(err)
  })
}

function save () {
  mainWindow.webContents.executeJavaScript('save()')
}
/* jamais appelée
function copyCodeBase64 () {
  mainWindow.webContents.executeJavaScript('copyCodeBase64()')
} /* */

function getLanguage () {
  language = app.getLocale().substring(0, 2)
  if ((language !== 'fr') && (language !== 'es') && (language !== 'en')) language = 'en'
}

function getStr (ch) {
  switch (language) {
    case 'fr' :
      return textFr[ch]
    case 'en' :
      return textEn[ch]
    case 'es' :
      return textEs[ch]
  }
}
