/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

//
const NatObj = require('./types/NatObj')
const NatCal = require('./types/NatCal')
const Color = require('./types/Color')
const Dimf = require('./types/Dimf')
const InfoProx = require('./types/InfoProx')
const Nat = require('./types/Nat')
const Point = require('./types/Point')
const Pointeur = require('./types/Pointeur')
const kernel = require('./kernel/kernel')
const CListeObjets = require('./objets/CListeObjets')
const CMathGraphDoc = require('./objets/CMathGraphDoc')
const DataInputStream = require('./entreesSorties/DataInputStream')
const CalcR = require('./kernel/CalcR')
const CalcC = require('./kernel/CalcC')
const StyleTrait = require('./types/StyleTrait')

require('../css/mtg32Player.css')

module.exports = MtgAppLecteur

/**
 * Classe permettant de gérer l'affichage de figures MathGraph32 en JavaScript
 * @constructor
 * @param {boolean} bLoadTextes  true si on doit charger les textes suivant la langue
 * et faux sinon. true par défaut.
 * Lorsqu'une application mtgApp crée une instance de MtgAppLecteur (pour j3p par exemple)
 * il ne faux pas recharger kernel.textes car cela a dékà été fait par mtgApp.
 * @returns {MtgAppLecteur}
 */
function MtgAppLecteur (bLoadTextes) {
  this.docs = []
  var lt = arguments.length === 0 ? true : bLoadTextes
  if (lt) {
    const s = kernel.userLan()
    switch (s) {
      case 'fr' :
        kernel.textes = require('./textes/textesFr')
        break
      case 'es' :
        kernel.textes = require('./textes/textesEs')
        break
      default :
        kernel.textes = require('./textes/textesEn')
    }
  }
  var t = this
  // On mémorise les gestioonaires de souris et de touch pour pouvoir éventuellement les détruire
  this.onmdown = function (evt) { t.onmousedown(evt) }
  this.onmmove = function (evt) { t.onmousemove(evt) }
  this.onmup = function (evt) { t.onmouseup(evt) }
  this.ontstart = function (evt) { t.ontouchstart(evt) }
  this.ontmove = function (evt) { t.ontouchmove(evt) }
  this.ontend = function (evt) { t.ontouchend(evt) }
  this.ontcancel = function (evt) { t.ontouchcancel(evt) }
}

/*
MtgAppLecteur.prototype.start = function(callBack) {
  if (kernel.textes === undefined) {
    textLoadLecteur.initTextes(callBack);
  }
  else callBack();
}
*/

/**
 * Ajoute le document par son id représenté par une chaîne chdoc en dataBase64
 * @param {String} idDoc  L'id du comument ui sera celle du svg contenant la figure.
 * @param {String} chdoc  Chaine base64 décrivant la figure (obtenue via MathGraph32 par
 * le menu Edition - Copier le code de la figure (pour html)
 * @param {boolean} displayOnLoad  true si la figure doit être affichée dès le démarrage de la page.
 * @param {boolean} isActive  true si la figure répond dès le démarrage aux actions souris et clavier.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.addDoc = function (idDoc, chdoc, displayOnLoad = true, isActive = true) {
  var svg, liste, li, i, nb
  svg = document.getElementById(idDoc)
  if (svg === null) return
  if (svg.localName !== 'svg') return

  // Version 6.4 : doit avoir le même fontFamily sur le conteneur svg que celui de body
  // pour éviter ce bug MathJax https://github.com/mathjax/MathJax/issues/2257
  try {
    const rootFontFamily = getComputedStyle(document.body).fontFamily
    const containerFontFamily = getComputedStyle(svg.parentNode).fontFamily
    if (rootFontFamily && rootFontFamily !== containerFontFamily) {
      console.log(`dans addDoc body a la police ${rootFontFamily} que l’on impose à notre container pour éviter un bug MathJax (il avait ${containerFontFamily})`)
      svg.parentNode.style.fontFamily = rootFontFamily
    }
  } catch (error) {
    console.error('plantage en essayant d’imposer fontFamily au conteneur dans addDoc', error)
  }

  // Modification version 6.3.2 pour rendre le player compatible avec les figures avec hachures.
  var defs = document.getElementById('mtg32_patterns')
  if (defs === null) {
    defs = kernel.cens('defs', {
      id: 'mtg32_patterns'
    })
    svg.appendChild(defs)
  }
  // Fin modif 6.3.2
  // On rend tous les éléments du div parent non sélectionnables
  const par = svg.parentNode
  // Ajout version mtgApp. Etait auparavant dans CEditeurFormule
  if (par.style.position === '') par.style.position = 'relative'
  // Ajouts versions 6.4 car MathJax utilisera les fontes du div parnet
  // $(par).css("font-weight", "normal").css("font-family", "\"Times New Roman\", Times, serif;");  // Inutile fait en css par la class  svgMtg

  // kernel.disableSelection(par);
  par.setAttribute('class', 'text2jax_ignore') // Ajout version 5.0.3
  var doc = new CMathGraphDoc(idDoc, displayOnLoad, isActive)
  this.docs.push(doc)
  var ba = kernel.base64Decode(chdoc)
  var inps = new DataInputStream(ba)
  doc.read(inps)

  this.prepareTracesEtImageFond(svg, doc)
  liste = new CListeObjets()
  liste.ajoutePointsPunaises(doc.listePr)
  doc.listeARecalculer = new CListeObjets()
  // Spécial JavaScript. Chaque document aura un membre pointCapture
  doc.listeExclusion = liste
  var winit = parseInt(svg.getAttribute('width'))
  var hinit = parseInt(svg.getAttribute('height'))
  doc.dimf = new Dimf(winit, hinit)
  // Modifié version 6.3.0
  // On crée un divdestiné à contenir le svg servant à des affichages auxiliaires
  /*
  */

  // On modifie si-nécessaire la dimension du svg destiné à afficher provisoirement les affichages de texte
  // Si c'est la première figure, on le crée.
  var svgAux = document.getElementById('mtgSvgAux')
  if (svgAux === null) {
    var divSvgAux = kernel.ce('div', {
      position: 'absolute',
      style: 'position:absolute;left:0px;top:0px;width:' + winit + 'px;height:' + hinit + 'px;pointer-events:none'
    })
    par.appendChild(divSvgAux)

    svgAux = kernel.cens('svg', {
      id: 'mtgSvgAux',
      position: 'absolute',
      top: 0,
      left: 0,
      width: winit,
      height: hinit,
      visibility: 'hidden'
    })
    divSvgAux.appendChild(svgAux)
  } else {
    /*
    var w = parseInt(svgAux.getAttribute("width"));
    var h = parseInt(svgAux.getAttribute("height"));
    if (w < winit) svgAux.setAttribute("width", winit);
    if (h < hinit) svgAux.setAttribute("height", hinit);
    */
    svgAux.setAttribute('width', winit)
    svgAux.setAttribute('height', hinit)
  }

  svg.addEventListener('mousedown', this.onmdown, false)
  svg.addEventListener('mousemove', this.onmmove, false)
  svg.addEventListener('mouseup', this.onmup, false)
  svg.addEventListener('touchstart', this.ontstart, false)
  svg.addEventListener('touchmove', this.ontmove, false)
  svg.addEventListener('touchend', this.ontend, false)
  svg.addEventListener('touchcancel', this.ontcancel, false)
  // Pour chaque variable associée à un affichage de valeur avec boutons plus et moins on crée un div
  li = doc.listePr
  nb = 0
  for (i = li.longueur() - 1; i >= 0; i--) {
    var el = li.get(i)
    if (el.getNatureCalcul() === NatCal.NVariable) {
      if (el.dialogueAssocie) {
        el.creeDiv(nb)
        nb++
      }
    }
  }
}

MtgAppLecteur.prototype.prepareTracesEtImageFond = function (svg, doc) {
  var rect = kernel.cens('rect', {
    width: '100%',
    height: '100%',
    fill: doc.couleurFond.rgb()
  })
  // Ligne suivante importante : Ce rectangle ne doit pas réagir aux événements souris
  // Car sinon il y a problème lors de l'appel de display
  rect.style.pointerEvents = 'none'
  svg.appendChild(rect)

  doc.gTraces = kernel.cens('g', {
    id: doc.idDoc + 'Traces'
  })
  svg.appendChild(doc.gTraces)
  if (doc.imageFond !== null) {
    var g = kernel.cens('image', {
      x: '0',
      y: '0',
      width: doc.widthImageFond,
      height: doc.heightImageFond,
      'pointer-events': 'none' // Ajout version 6.3.4
    })
    g.setAttributeNS('http://www.w3.org/1999/xlink', 'href', 'data:image/png;base64,' + kernel.base64Encode(doc.imageFond, true))
    svg.insertBefore(g, doc.gTraces)
  }
}

/**
 * Fonction retirant le document d'id idDoc, retirant les gestionnaires d'événements qui lui sont associés
 * et effaçant touts les éléments graphiques du svg associé
 * @param {String} idDoc  L'id du document à retirer.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.removeDoc = function (idDoc) {
  var svg, list, ind, i, el
  var doc = this.getDoc(idDoc)
  if (doc !== null) {
    list = doc.listePr
    svg = document.getElementById(idDoc)
    if ((svg !== null) && (svg.localName === 'svg')) {
      // Modification version 6.3.5 : On met d'abord la variable hasgElement de tous les objets graphiques à false car
      // une fonction de callback peut encore être appelée demandant un update des éléments graphiques
      for (i = 0; i < list.longueur(); i++) {
        el = list.get(i)
        if (el.estDeNature(NatObj.NTtObj)) {
          el.hasgElement = false
        }
      }
      while (svg.childNodes.length !== 0) svg.removeChild(svg.childNodes[0])
      svg.removeEventListener('mousedown', this.onmdown, false)
      svg.removeEventListener('mousemove', this.onmmove, false)
      svg.removeEventListener('mouseup', this.onmup, false)
      svg.removeEventListener('touchstart', this.ontstart, false)
      svg.removeEventListener('touchmove', this.ontmove, false)
      svg.removeEventListener('touchend', this.ontend, false)
      svg.removeEventListener('touchcancel', this.ontcancel, false)
      list.deleteComponents() // Pour retirer les éditeurs et les boutons associés aux variables
    }
    list.retireTout()
    ind = this.indexOfDoc(idDoc)
    this.docs.splice(ind, 1)
  }
}

/**
 * Fonction retirant tous les documents et effaçant le contenu de tous les svg contenant des figures.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.removeAllDoc = function () {
  // this.docs.forEach(doc => this.removeDoc(doc.idDoc))
  // Corrigé par Yves. Le ForEach ici ne marche pas car la fonction removeDoc retire l'élement du tableau.
  // Si, par exemple, on a deux documents dans la liste il en reste un au retour...
  while (this.docs.length > 0) this.removeDoc(this.docs[0].idDoc)
}

/**
 * Fonction recalculant toutes les figures et les affichant.
 * @param {boolean} brandom  Si true, les calculs aléatoires pas la fonction rand()
 * sont réactualisés.
 * Modifié pour qu'au premier démarrage on charge les textes de la langue utilisée
 * @returns {undefined}
 */
/*
MtgAppLecteur.prototype.calculateAndDisplayAll = function(brandom) {
  if (kernel.textes === undefined) {
    var self = this;
    textLoadLecteur.initTextes(function() {
      self.endCalculateAndDisplayAll(brandom)
    });
  }
  else this.endCalculateAndDisplayAll(brandom);
};
*/
// Modifié version 6.4.5
MtgAppLecteur.prototype.calculateAndDisplayAll = function (brandom) {
  for (let i = 0; i < this.docs.length; i++) {
    const doc = this.docs[i]
    const idDoc = doc.idDoc
    this.calculateFirstTime(idDoc, brandom)
    this.display(idDoc)
  }
}

/**
 * Fonction de callback appelée lorsque MathJax a traité toues les formules
 * en LaTeX et que la figure est prête pour affichage.
 * @param {CMathGraphDoc} doc  le document à afficher.
 * @param {svg} svg  Le svg contenant la figure.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.callBack = function (svg, doc) {
  const liste = doc.listePr
  liste.afficheTout(0, svg, true, doc.couleurFond, doc.opacity)
}
/**
 * Fonction demandant la validation de tous les éditeurs de formule de la figure associée
 * au document d'id idDoc.
 * Si la formule qu'ils contiennent est incorrecte ils se trouvent encadrés de rouge.
 * Renvoie true uniquement si aucun éditeur de formule ne contient de faute de syntaxe.
 * @param {type} idDoc
 * @returns {boolean}
 */
MtgAppLecteur.prototype.fieldsValidation = function (idDoc) {
  return this.getDoc(idDoc).listePr.fieldsValidation()
}
/**
 * Fonction renvoyant true si le premier champ d'édition de la figure associée au document d'id idDoc
 * associé au calcul nomCalcul est non vide.
 * @param {String} idDoc  l'id du document contenant la figure interrogée.
 * @param {String} nomCalcul  Le nom du calcul auquel est associé l'éditeur.
 * @returns {boolean}
 */
MtgAppLecteur.prototype.fieldValidation = function (idDoc, nomCalcul) {
  return this.getDoc(idDoc).listePr.fieldValidation(nomCalcul)
}
/**
 * Fonction renvoyant le contenu du premier éditeur de formule asssocié au calcul nomCalcul
 * dans le dcument d'id idDoc.
 * @param {String} idDoc
 * @param {type} nomCalcul
 * @returns {String}
 */
MtgAppLecteur.prototype.getFieldValue = function (idDoc, nomCalcul) {
  return this.getDoc(idDoc).listePr.getFieldValue(nomCalcul)
}
/**
 * Fonction activant le premier éditeur de formule contenant une
 * formule qui n'est pas correcte sur le plan syntaxique dans le document d'id idDoc.
 * @param {String} idDoc
 * @returns {undefined}
 */
MtgAppLecteur.prototype.activateFirstInvalidField = function (idDoc) {
  this.getDoc(idDoc).listePr.activateFirstInvalidField()
}
/**
 * Fonction activant le premier éditeur de formule vide ou contenant une
 * formule qui n'est pas correcte sur le plan syntaxique dans le document d'id idDoc
 * @param {String} idDoc
 * @returns {undefined}
 */
MtgAppLecteur.prototype.activateFirstInvalidOrEmptyField = function (idDoc) {
  this.getDoc(idDoc).listePr.activateFirstInvalidOrEmptyField()
}
/**
 * Fonction donnant le focus au premier éditeur de formule qui ne contient rien
 * dans le document d'id idDoc
 * @param {String} idDoc
 * @returns {undefined}
 */
MtgAppLecteur.prototype.activateFirstEmptyField = function (idDoc) {
  this.getDoc(idDoc).listePr.activateFirstEmptyField()
}
/**
 * Fonction donnant le focus au premierr éditeur de formule asscoié au calcul
 * ou à la fonction nommé nomCalcul dans le document d'id idDoc.
 * @param {String} idDoc
 * @param {String} nomCalcul
 * @returns {undefined}
 */
MtgAppLecteur.prototype.activateField = function (idDoc, nomCalcul) {
  this.getDoc(idDoc).listePr.activateField(nomCalcul)
}
/**
 * Fonction donnant à l'élément graphique du document d'id idDoc et dont le n° d'identification html est
 * ind la couleur donnée par ses composantes RVB (entiers compris entre 0 et 255)
 * @param {string} idDoc
 * @param {int} ind
 * @param {int} r  Composante rouge de la couleur (entre 0 et 255).
 * @param {int} v  Composante verte de la couleur (entre 0 et 255).
 * @param {int} b  Composante bleue de la couleur (entre 0 et 255).
 * @param {boolean} bImmediat : Si true, réaffichage immédiat de l'objet dont on a changé la couleur
 * @returns {boolean} true si tout s'est bien passé, false sinon
 */

/**
 * Fonction servant à rendre visible ou masquer l'élément graphique du document d'id idDoc et dont le n° d'identification html est
 * ind
 * @param {string} idDoc
 * @param {string|int} ind : N° d'identification html de l'élément (entier) ou chaîne de caractères
 * commençant par # et suivie du tag de l'élément
 * @param {boolean} bVisible : true pour rendre l'élément visible, false pour le masquer
 * @param {boolean} bImmediat : Si true, réaffichage ou masquage immédiat de l'objet
 * @return {boolean} true si tout s'est bien passé, false sinon
 */
MtgAppLecteur.prototype.setVisible = function (idDoc, ind, bVisible, bImmediat = true) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const liste = doc.listePr
  const el = liste.getById(ind)
  if (el === null || !el.existe || !el.estDeNature(NatObj.NTtObj)) return false
  el.masque = !bVisible
  if (bImmediat) {
    MathJax.Hub.Queue(function () {
      const svg = document.getElementById(idDoc)
      liste.callBack(el, svg, doc.couleurFond, doc.opacity, true)
    })
  }
  return true
}

/**
 * Fonction donnant à l'élément d'indice html ind dans le document d'id idDoc le texte txt.
 * Cet élément peut être soit un CLatex soit un CCommentaire
 * @param {string} idDoc
 * @param {string|int} ind : N° d'identification html de l'élément (entier) ou chaîne de caractères
 * commençant par # et suivie du tag de l'élément
 * @param {string} txt
 * @param {boolean} bImmediat : Si true, réaffichage de l'objet dont on a changé la couleur
 * @returns {boolean} true si tout s'est bien passé, false sinon
 */
MtgAppLecteur.prototype.setText = function (idDoc, ind, txt, bImmediat = true) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const list = doc.listePr
  const el = list.getById(ind)
  if (el === null || !el.estDeNature(Nat.or(NatObj.NCommentaire, NatObj.NLatex))) return false
  el.chaineCommentaire = txt
  if (bImmediat) {
    const svg = document.getElementById(idDoc)
    el.positionne(false)
    el.setReady4MathJax()
    MathJax.Hub.Queue(function () {
      // list.callBack(el, svg, doc.couleurFond, doc.opacity, true)
      if (el.g) el.replaceg(svg, doc.couleurFond, doc.opacity)
    })
  }
  return true
}

/**
 * Fonction zoomant tous les éléments de la figure d'id idDoc par rapport au point de coordonnées (x; y) avec le rapport de zoom ratio
 * @param {string} idDoc : L'id de la figure
 * @param {number} x abscisse du centre du zoom
 * @param {number} y ordonnée du centre du zomm
 * @param {number} ratio : rapport du zoom
 * @param {boolean} bImmediat : Si true, réaffichage de l'objet dont on a changé la couleur
 * @returns {boolean} true si tout s'est bien passé, false sinon
 */
MtgAppLecteur.prototype.zoom = function (idDoc, x, y, ratio, bImmediat = true) {
  const doc = this.getDoc(idDoc)
  if (doc === null || ratio <= 0) return false
  const list = doc.listePr
  list.zoom(x, y, ratio)
  if (bImmediat) {
    const svg = document.getElementById(idDoc)
    const dimf = new Dimf(parseInt(svg.getAttribute('width')), parseInt(svg.getAttribute('height')))
    list.positionne(false, dimf)
    list.update(svg, doc.couleurFond, doc.opacity, true, true)
  }
  return true
}

/**
 * Fonction translatant tous les objets de la figure d'id idDoc d'un vecteur (decx, decy)
 * @param {string} idDoc : L'id de la figure
 * @param {number} decx : Première coordonnnée du vecteur de la translation
 * @param {number} decy : Deuxième coordonnnée du vecteur de la translation
 * @param {boolean} bImmediat Si true, réaffichage de l'objet dont on a changé la couleur
 * @return {boolean} : true si tout s'est bien passé, false sinon
 */
MtgAppLecteur.prototype.translate = function (idDoc, decx, decy, bImmediat = true) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const list = doc.listePr
  const modif = list.translateDe(decx, decy)
  if (bImmediat && modif) {
    const svg = document.getElementById(idDoc)
    const dimf = new Dimf(parseInt(svg.getAttribute('width')), parseInt(svg.getAttribute('height')))
    list.positionne(false, dimf)
    list.update(svg, doc.couleurFond, doc.opacity, true, true)
  }
  return true
}

/*
MtgAppLecteur.prototype.addFreePoint = function (idDoc, x, y, nom = '', red = 0, green = 0, blue = 0, motif = 3, bNomMasque = false, tailleNom = 16, decx = 0, decy = 3) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const list = doc.listePr
  const svg = document.getElementById(idDoc)
  const dimf = new Dimf(parseInt(svg.getAttribute('width')), parseInt(svg.getAttribute('height')))
  const pb = new CPointBase(list, null, false, new Color(red, green, blue), bNomMasque, decx, decy, false, nom, tailleNom, motif, false, true, x, y)
  list.add(pb)
  pb.positionne(false, dimf)
  pb.id = list.id + '#' + pb.index
  MathJax.Hub.Queue(function () {
    pb.creeAffichage(svg, false, doc.couleurFond, doc.opacity)
  })
  return true
}
 */

/**
 * Fonction remmettant à jour la figure du document d'id idDoc après avoir modifié certains de ses objets
 * directement (par exemple en utilisant translatePoint()
 * @param {string} idDoc : L'id du document à réafficher
 * @return {boolean} : true si tout s'est bien passé, false sinon
 */
MtgAppLecteur.prototype.updateFigure = function (idDoc) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const list = doc.listePr
  const svg = document.getElementById(idDoc)
  const dimf = new Dimf(parseInt(svg.getAttribute('width')), parseInt(svg.getAttribute('height')))
  list.positionne(false, dimf)
  list.update(svg, doc.couleurFond, doc.opacity, true, true)
  return true
}

/**
 * Fonction déplaçant un point libre aux coordonnées (x;y) dans le svg du document d'id idDoc
 * @param {string} idDoc
 * @param {string|int} name : Le nom du point ou # suivi du tag du point ou son indice html (entier)
 * @param {number} x / Nouvelle abscisse à donner au point
 * @param {number} y
 * @param {boolean} bImmediat : Si ture la figure est tout de suite réaffichée
 * @return {boolean} : true si tout s'est bien passé, false sinon
 */
MtgAppLecteur.prototype.setPointPosition = function (idDoc, name, x, y, bImmediat = true) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const list = doc.listePr
  let pt = null
  if (typeof name !== 'string') {
    pt = list.getById(parseInt(name))
  } else {
    if (name.charAt(0) === '#') pt = list.getByTag(name.substring(1))
    else pt = list.getPointByName(name)
  }
  if (pt === null || !pt.estDeNature(NatObj.NPointBase)) return false
  pt.x = x
  pt.y = y
  if (bImmediat) {
    const svg = document.getElementById(idDoc)
    const dimf = new Dimf(parseInt(svg.getAttribute('width')), parseInt(svg.getAttribute('height')))
    list.positionneDependantsDe(false, dimf, pt)
    list.updateDependants(pt, svg, doc.couleurFond, doc.opacity, true)
  }
}

/**
 * Fonction translatant un point libre du vecteur de coordonnées (deltax; deltay) dans le svg du document d'id idDoc
 * @param {string} idDoc
 * @param {string|int} name : Le nom du point ou # suivi du tag du point ou son indice html (entier)
 * @param {number} deltax : Valeur à ajouter à l'abscisse du point
 * @param {number} deltay : Valeur à ajouter à l'ordonnée du point
 * @param {boolean} bImmediat : Si ture la figure est tout de suite réaffichée
 * @return {boolean} : true si tout s'est bien passé, false sinon
 */
MtgAppLecteur.prototype.translatePoint = function (idDoc, name, deltax, deltay, bImmediat = true) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const list = doc.listePr
  let pt = null
  if (typeof name !== 'string') {
    pt = list.getById(parseInt(name))
  } else {
    if (name.charAt(0) === '#') pt = list.getByTag(name.substring(1))
    else pt = list.getPointByName(name)
  }
  if (pt === null || !pt.estDeNature(NatObj.NPointBase)) return false
  pt.x += deltax
  pt.y += deltay
  if (bImmediat) {
    const svg = document.getElementById(idDoc)
    const dimf = new Dimf(parseInt(svg.getAttribute('width')), parseInt(svg.getAttribute('height')))
    list.positionne(false, dimf)
    list.update(svg, doc.couleurFond, doc.opacity, true, true)
  }
}

/**
 * Fonction renvoyant les coordonnées actuelles du point nommé name relativement au SVG
 * du document d'id idDoc
 * @param {string} idDoc : L'id du document
 * @param {string|int} name : Le nom du point ou # suivi du tag du point ou son indice html (entier)
 * @return {null|{x: *, y: *}}
 */
MtgAppLecteur.prototype.getPointPosition = function (idDoc, name) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return null
  const list = doc.listePr
  let pt = null
  if (typeof name !== 'string') {
    pt = list.getById(parseInt(name))
  } else {
    if (name.charAt(0) === '#') pt = list.getByTag(name.substring(1))
    else pt = list.getPointByName(name)
  }
  if (pt === null || !pt.estDeNature(NatObj.NTtPoint)) return null
  return { x: pt.x, y: pt.y }
}
/**
 * Fonction donnant à l'élément d'id ind dans le document d'id idDoc le style de trait
 * style et l'épaisseur thickness
 * @param {string} idDoc
 * @param {string|int} ind : N° d'identification html de l'élément (entier) ou chaîne de caractères
 * commençant par # et suivie du tag de l'élément
 * @param {string|number|null} style entier de 0 à 5. O pour trait continu, 1 pour pointillés etc Voir palette de couleurs de mtg32
 *                         ou chaine vide ou null si on ne veut changer que l'épaisseur de l'objet
 * @param {string|number|null} thickness entier de 1 à 12 donnant l'épaisseur du trait
 *                         ou chaine vide ou null si on ne veut changer que le style
 * @param {boolean} bImmediat  Si true, le réaffichage est immédiat
 * @return {boolean}
 */
// Fonction ajoutée version 6.5.2 pour utilisation externe
MtgAppLecteur.prototype.setLineStyle = function (idDoc, ind, style, thickness, bImmediat = true) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const list = doc.listePr
  const el = list.getById(ind)
  if (el === null || !el.existe || !el.estDeNature(NatObj.NObjLigne)) return false
  const st = (style === null || style === '') ? el.style.style : parseInt(style)
  const th = (thickness === null || thickness === '') ? el.style.strokeWidth : parseInt(thickness)
  if (st < 0 || st > 5 || th <= 0 || th > 12) return false
  el.style = new StyleTrait(list, st, th)
  if (bImmediat && !el.masque) {
    // const svg = document.getElementById(idDoc)
    MathJax.Hub.Queue(function () {
      if (el.g) el.setgLineStyle(el.style)
    })
  }
  return true
}

// Fonction ajoutée version 6.5.2
/**
 * Fonction donnant à l'élément d'id ind dans le document d'id idDoc la couleur de composantes r, v, b
 * style et l'épaisseur thickness
 * @param {string} idDoc
 * @param {string|int} ind : N° d'identification html de l'élément (entier) ou chaîne de caractères
 * commençant par # et suivie du tag de l'élément
 * @param {string} r entier donnant la composante rouge de la couleur
 * @param {string} v entier donnant la composante verte de la couleur
 * @param {string} b entier donnant la composante bleue de la couleur
 * @param {boolean} bImmediat  Si true, le réaffichage est immédiat
 * @return {boolean}
 */

MtgAppLecteur.prototype.setColor = function (idDoc, ind, r, v, b, bImmediat = true) {
  const color = new Color(r, v, b)
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const liste = doc.listePr
  const el = liste.getById(ind)
  if (el === null || !el.existe || !el.estDeNature(NatObj.NTtObj)) return false
  el.donneCouleur(color)
  if (bImmediat && !el.masque) {
    const svg = document.getElementById(idDoc)
    const dimf = new Dimf(parseInt(svg.getAttribute('width')), parseInt(svg.getAttribute('height')))
    el.positionne(false, dimf)
    el.setReady4MathJax()
    /* Modification version 6.4.1
    MathJax.Hub.Queue([liste.callBack, el, svg, doc.couleurFond, doc.opacity, true])
    */
    MathJax.Hub.Queue(function () {
      // Modif  version 6.5.2
      // liste.callBack(el, svg, doc.couleurFond, doc.opacity, true)
      // el.replaceg(svg, doc.couleurFond, doc.opacity)
      if (el.g) el.setgColor()
    })
  }
  return true
}

/**
 * Fonction renvoyant le composant SVG  de l'élément d'id ind dans le document d'id idDoc
 * Renvoie null s'il n'y a pas d'élément d'indice ind ou s'il n'existe pas où n'est pas affiché
 * @param {string} idDoc
 * @param {string|int} ind : N° d'identification html de l'élément (entier) ou chaîne de caractères
 * commençant par # et suivie du tag de l'élément
 * @return {null|*}
 C:\Users\yvesb\.gradle
 C:\Projets_JavaScript\MathGraphJS\mathgraph_js
 C:\Users\yvesb\.WebStorm2019.2\system
 */
MtgAppLecteur.prototype.getSVGElement = function (idDoc, ind) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return null
  const el = doc.listePr.getById(ind)
  if (el === null || !el.estDeNature(NatObj.NTtObj) || !el.g) return null
  return el.g
}

/**
 * Fonction associant un eventeListener au svg Element représentant l'objet d'indice html ind
 * dans la figure d'id idDoc
 * @param {string} idDoc : L'id de la figure
 * @param {string|int} ind : N° d'identification html de l'élément (entier) ou chaîne de caractères
 * commençant par # et suivie du tag de l'élément
 * @param {string} eventName : Le nom de l'événement (par exemple mousemove)
 * @param {function} callBack : La fonction de callBack de paramètre l'événement appelant
 * @return {boolean} : true si tout s'est bien passé, false sinon
 */
MtgAppLecteur.prototype.addEventListener = function (idDoc, ind, eventName, callBack) {
  // Il faut employer la pile de MathJax car quand on affiche la figure les svg elements de la figure
  // ont par défaut un attribut pointer-events à 'none'
  // Ce n'est qu'uen fois tout affiché que l'on peut modifier l'attribut pointer-events ce qui permet
  // ensuite d'affecter un event listener à cet objet SVG
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const el = doc.listePr.getById(ind)
  if (el === null || !el.estDeNature(NatObj.NTtObj)) return false
  MathJax.Hub.Queue(function () {
    const gel = el.g
    if (gel) {
      gel.setAttribute('pointer-events', 'all')
      el.pointerevents = 'all'
      // Si le svg element de l'objet est un g element contenant d'autres objets il faut aussi qu'ils réagissent
      // aux événements souris
      for (let i = 0; i < gel.childNodes.length; i++) {
        gel.childNodes[i].setAttribute('pointer-events', 'all')
      }
      if (!el.cbmap) el.cbmap = new Map()
      else {
        const oldcb = el.cbmap.get(eventName)
        if (oldcb) {
          gel.removeEventListener(eventName, oldcb)
        }
      }
      const cb = function (ev) {
        let pos
        const id = el.listeProprietaire.id
        const svg = document.getElementById(id)
        if (eventName.indexOf('touch') === 0) {
          pos = kernel.getTouchPositionToParent(ev, svg)
        } else {
          pos = kernel.getMousePositionToParent(ev, svg)
        }
        callBack(ev, pos[0], pos[1])
      }
      el.cbmap.set(eventName, cb)
      gel.addEventListener(eventName, cb)
    }
  })
  return true
}

/**
 * Fonction retirant l'eventListener associé à svg element représentant l'objet d'indice html ind
 * dans le document d'id idDoc
 * @param {string} idDoc : L'id du document
 * @param {string|int} ind : N° d'identification html de l'élément (entier) ou chaîne de caractères
 * commençant par # et suivie du tag de l'élément
 * @param {string} eventName : Le nom de l'événement (par exemple mousemove)
 * @return {boolean} : true si tout s'est bien passé, false sinon
 */
MtgAppLecteur.prototype.removeEventListener = function (idDoc, ind, eventName) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const el = doc.listePr.getById(ind)
  if (el === null || !el.estDeNature(NatObj.NTtObj) || !el.g || !el.cbmap || !el.cbmap.get(eventName)) return false
  const callBack = el.cbmap.get('eventName')
  el.cbmap.delete(eventName)
  const gel = el.g
  if (el.cbmap.size === 0) {
    el.pointerevents = 'none'
    gel.setAttribute('pointer-events', 'none')
    // Si le svg element de l'objet est un g element contenant d'autres objets il faut aussi qu'ils ne réagissent pas
    // aux événements souris
    for (let i = 0; i < gel.childNodes.length; i++) {
      gel.childNodes[i].setAttribute('pointer-events', 'none')
    }
  }
  try {
    gel.removeEventListener(eventName, callBack)
  } catch (e) {
    return false
  }
  return true
}

MtgAppLecteur.prototype.addCallBackToSVGListener = function (idDoc, eventName, callBack) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  // L'objet doc peut contenir un objet map formé d'entrées sont le premier élément est un nom d'événement souris
  // et le deuxième une fonction de callBack à appeler une fois exécuté le processus normal du player mtg32
  if (!doc.cbmap) {
    doc.cbmap = new Map()
  }
  doc.cbmap.set(eventName, callBack)
  return true
}

MtgAppLecteur.prototype.removeCallBackToSVGListener = function (idDoc, eventName) {
  const doc = this.getDoc(idDoc)
  if (doc === null || !doc.cbmap) return false
  return doc.cbmap.delete(eventName)
}

/**
 * Fonction vidant tous les éditeurs de formule de la figure d'id idDoc
 * @param {String} idDoc
 * @returns {undefined}
 */
MtgAppLecteur.prototype.setEditorsEmpty = function (idDoc) {
  this.getDoc(idDoc).listePr.setEditorsEmpty()
}
/**
 * Fonction mettant dans l'éditeur la chaîne de caractères st et mettant à jour
 * en conséquence l'affichage de formule LaTeX s'il est activé, dans le document d'id idDoc
 * @param {String} idDoc
 * @param {String} nomCalcul
 * @param {String} st
 * @returns {undefined}
 */
MtgAppLecteur.prototype.setEditorValue = function (idDoc, nomCalcul, st) {
  this.getDoc(idDoc).listePr.setEditorValue(nomCalcul, st)
}
/**
 * Fonction attribuant à un éditeur de formule une chaîne de caractères contenant
 * les caractères autorisés.
 * Si cette chaîne est vide, tous les caractères seront autorisés.
 * @param {type} idDoc
 * @param {type} nomCalcul
 * @param {type} st
 * @returns {undefined}
 */
MtgAppLecteur.prototype.setEditorCharset = function (idDoc, nomCalcul, st) {
  var liste = this.getDoc(idDoc).listePr
  for (var i = 0; i < liste.longueur(); i++) {
    var elb = liste.get(i)
    if (elb.estDeNature(NatObj.NEditeurFormule) && (elb.calculAssocie.nomCalcul === nomCalcul)) {
      elb.charset = st
      break
    }
  }
}
/**
 * Fonction associant, dans le document d'id idDoc, au premier éditeur associé à nomCalcul
 * une fonction de callBack qui sera appelée quand l'utilisateur valide par OK le contenu de l'éditeur.
 * @param {String} idDoc
 * @param {String} nomCalcul
 * @param {function} f  La fonction de callBack
 * @returns {undefined}
 */
MtgAppLecteur.prototype.setEditorCallBackOK = function (idDoc, nomCalcul, f) {
  this.getDoc(idDoc).listePr.setEditorCallBackOK(nomCalcul, f)
}
/**
 * Renvoie la valeur actuelle du calcul réel nommé nomCalcul dans le document d'id idDoc.
 * Renvoie -1 si le calcul n'existe pas.
 * @param {String} idDoc
 * @param {String} nomCalcul
 * @returns {number}
 */
MtgAppLecteur.prototype.valueOf = function (idDoc, nomCalcul) {
  return this.getDoc(idDoc).listePr.valueOf(nomCalcul)
}
/**
 * Fonction changeant, dans le document d'id idDoc, la formule du calcul ou de
 * la fonction (réelle ou complexe) de nom nomCalcul.
 * La nouvelle formule est contenue dans la chaîne de caractères formule.
 * Renvoie true si la formule était valide et false sinon.
 * @param {String} idDoc
 * @param {String} nomCalcul
 * @param {String} formule
 * @returns {undefined}
 */
MtgAppLecteur.prototype.giveFormula2 = function (idDoc, nomCalcul, formule) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) this.getDoc(idDoc).listePr.giveFormula2(nomCalcul, formule)
}
/**
 * Renvoie la formule en ligne (avec des *) du calcul ou de la fonction nommé nomCalcul
 * contenue dans le document d'id idDoc.
 * @param {String} idDoc
 * @param {String} nomCalcul
 * @returns {String}
 */
MtgAppLecteur.prototype.getFormula = function (idDoc, nomCalcul) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) return this.getDoc(idDoc).listePr.getFormula(nomCalcul)
  return ''
}
// Ajout version 4.9.3
/**
 * Fonction renvoyant la formule LaTeX représentant le calcul ou la fonction
 * dont le nom est nomCalcul dans le document d'id idDoc.
 * @param {String} idDoc
 * @param {String} nomCalcul
 * @returns {String}
 */
MtgAppLecteur.prototype.getLatexFormula = function (idDoc, nomCalcul) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) return this.getDoc(idDoc).listePr.getLatexFormula(nomCalcul)
  return ''
}
/**
 * Fonction donnant à tous les éditeur de formule de la fiure d'id idDoc
 * la taille size.
 * Utilisé dans j3p.
 * @param {String} idDoc
 * @param {String} size
 * @returns {undefined}
 */
MtgAppLecteur.prototype.setEditorsSize = function (idDoc, size) {
  var doc, liste, i, el
  doc = this.getDoc(idDoc)
  if (doc !== null) {
    liste = this.getDoc(idDoc).listePr
    for (i = 0; i < liste.longueur(); i++) {
      el = liste.get(i)
      if (el.estDeNature(NatObj.NEditeurFormule)) {
        el.editeur.style.fontSize = size
        el.latex.xNom = el.xNom + el.divi.clientWidth
        el.latex.yNom = el.yNom + el.editeur.clientHeight / 2
      }
    }
  }
}
/**
 * Fonction recalculant la figure d'identifiant idDoc
 * @param {String} idDoc
 * @param {boolean} brandom
 * @returns {undefined}
 */
MtgAppLecteur.prototype.calculate = function (idDoc, brandom) {
  var doc = this.getDoc(idDoc)
  var svg = document.getElementById(idDoc)
  var dimf = new Dimf(parseInt(svg.getAttribute('width')), parseInt(svg.getAttribute('height')))
  // On appelle positionneFull pour que les tests d'équivalences et de factorisation soient recalculés
  if (doc !== null) doc.listePr.positionneFull(brandom, dimf)
}

/**
 * Fonction devant être appelée quand on calcule la figure pour la première fois avant de l'afficher
 * Dans le  cas où la figure contient une macro de démarrage qui doit elle même construire des objets supplémentaires, cette macro
 * est d'abord exécutée
 * @param {String} idDoc
 * @param {String} brandom
 */
MtgAppLecteur.prototype.calculateFirstTime = function (idDoc, brandom) {
  if (typeof brandom !== 'boolean') brandom = false
  const doc = this.getDoc(idDoc)
  if (doc !== null) {
    const liste = doc.listePr
    const svg = document.getElementById(idDoc)
    const dimf = new Dimf(parseInt(svg.getAttribute('width')), parseInt(svg.getAttribute('height')))
    const macroDem = liste.macroDemarrage
    if (macroDem !== null) {
      // Ajout version 5.1
      macroDem.setMacroLanceuse(null)
      //
      if (macroDem.className === 'CMacroSuiteMacros') {
        for (let j = 0; j < macroDem.listeAssociee.longueur(); j++) {
          macroDem.listeAssociee.get(j).execute(svg, dimf, doc.couleurFond, doc.opacity, false)
        }
      } else {
        // Version 4.9.7 : Ajout d'un dernier paramètre pour que la macro ne crée pas d'élements graphiques
        macroDem.execute(svg, dimf, doc.couleurFond, doc.opacity, false)
      }
    }
    // Modifié version 5.0.1 pour éviter des erreurs aléatoires sur les tests d'équivalences dans les exercices de calcul
    // doc.listePr.positionne(brandom, dimf)
    doc.listePr.positionneFull(brandom, dimf)
  }
}

/**
 * Fonction affichant la figure d'identifiant idDoc après avoir retiré tous
 * les éléments graphiques du svg de la figure.
 * @param {type} idDoc
 * @returns {undefined}
 */
MtgAppLecteur.prototype.display = function (idDoc) {
  const doc = this.getDoc(idDoc)
  const self = this
  if (doc !== null) {
    // On retire d'abord tous les enfants du svg
    const liste = doc.listePr
    const svg = document.getElementById(idDoc)

    MathJax.Hub.Queue(function () {
      // Attention il faut garder le defs servant à stocker les styles de hachures
      const defs = document.getElementById('mtg32_patterns')
      const defsParentNode = defs.parentNode
      while (svg.childNodes.length !== 0) {
        svg.removeChild(svg.childNodes[0])
      }
      if (defs !== null) {
        if (defsParentNode !== svg) defsParentNode.removeChild(defs) // Version 6.4.5 : on met toujours le defs dans la dernière figure ajoutée
        svg.appendChild(defs)
      }
      self.prepareTracesEtImageFond(svg, doc)
    })
    liste.setReady4MathJax()
    // MathJax.Hub.Queue(new function() {liste.afficheTout(0, document.getElementById(doc.idDoc), true, doc.couleurFond, doc.opacity)});
    /* Correction version WebPack
    var dimf = new Dimf(parseInt(svg.getAttribute("width")), parseInt(svg.getAttribute("height")));
    MathJax.Hub.Queue([this.callBack,doc,svg,dimf,true]);
    */
    /* Modifié version 6.4.1
    MathJax.Hub.Queue([this.callBack, svg, doc])
    */
    /* Modifié version 6.4.5 car la callBack apppelle elle-même MathJax.Hub.Queue et avec les promesses l'ordre d'appel n'est plus le bon
    MathJax.Hub.Queue(function () {
      self.callBack(svg, doc)
    })
    */
    // Affiche tout va appeler MathJax.Hub.Queue car il n'y a pas de dernier paramètrre
    liste.afficheTout(0, svg, true, doc.couleurFond, doc.opacity)
  }
}
/**
 * Fonction mettant à jour les éléments graphiques de la figure d'idDoc.
 * @param {type} idDoc
 * @returns {undefined}
 */
MtgAppLecteur.prototype.updateDisplay = function (idDoc) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) {
    var liste = doc.listePr
    var svg = document.getElementById(idDoc)
    liste.update(svg, doc.couleurFond, doc.opacity, true)
  }
}
/**
 * Fonction exécutant la macro d'intitulé nameMacro dans la
 * figure d'id idDoc.
 * @param {String} idDoc
 * @param {String} nameMacro
 * @returns {undefined}
 */
MtgAppLecteur.prototype.executeMacro = function (idDoc, nameMacro) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) doc.listePr.executeMacro(nameMacro)
}
/**
 * Renvoie l'indice dans le tableau this.docs du document d'id idDoc.
 * @param {String} idDoc
 * @returns {number}
 */
MtgAppLecteur.prototype.indexOfDoc = function (idDoc) {
  for (var i = 0; i < this.docs.length; i++) {
    if (this.docs[i].idDoc === idDoc) return i
  }
  return -1
}
/**
 * Renvoie le document d'id idDoc du tableau this.docs.
 * S'il n'y en a pas, renvoie null.
 * @param {String} idDoc
 * @returns {Array}
 */
MtgAppLecteur.prototype.getDoc = function (idDoc) {
  var ind = this.indexOfDoc(idDoc)
  return (ind === -1) ? null : this.docs[ind]
}
/**
 * Renvoie la liste CListeObjets contenant les objets du document d'id idDoc.
 * @param {String} idDoc
 * @returns {Array.listePr}
 */
MtgAppLecteur.prototype.getList = function (idDoc) {
  return this.getDoc(idDoc).listePr
}
/**
 * Fonction renvoyant le code LaTeX de l'affichage LaTex d'indice ind dans la liste
 * des objets créés (les indices commençant à zéro) dans le document d'id idDoc
 * @param {String} idDoc
 * @param {number} ind
 * @returns {String}
 */
MtgAppLecteur.prototype.getLatexCode = function (idDoc, ind) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) return doc.listePr.getLatexCode(ind)
  return ''
}

/**
 * Fonction utilisée pour les exercices en ligne et qui, s'il existe un affichage LaTeX cmmençant par la chaîne stratString,
 * renvoie le contenu cette chaîne tronqué de startString
 * @param idDoc
 * @param startString
 * @returns {*}
 */
MtgAppLecteur.prototype.getLatex = function (idDoc, startString) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) {
    var listePr = doc.listePr
    for (var i = 0; i < listePr.longueur(); i++) {
      var el = listePr.get(i)
      if (el.estDeNature(NatObj.NLatex)) {
        var ch = el.chaineCommentaire
        if (ch.indexOf(startString) !== -1) return ch.substring(startString.length)
      }
    }
  }
  return ''
}

/**
 * Rend la figure d'id idDoc réactive ou inative aux événements souris et clavier
 * suivant la valeur du boolean ba.
 * @param {String} idDoc
 * @param {boolean} ba
 * @returns {undefined}
 */
MtgAppLecteur.prototype.setActive = function (idDoc, ba) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) {
    doc.isActive = ba
    // Il faut aussi activer ou désactiver les champs d'édition de formule
    var liste = doc.listePr
    for (var i = 0; i < liste.longueur(); i++) {
      var el = liste.get(i)
      if (el.estDeNature(NatObj.NEditeurFormule)) {
        el.editeur.readOnly = !ba
      } else {
        if (el.getNatureCalcul() === NatCal.NVariable) {
          if (el.dialogueAssocie) {
            el.buttonplus.setAttribute('disabled', !ba)
            el.buttonmoins.setAttribute('disabled', !ba)
            el.buttonegal.setAttribute('disabled', !ba)
          }
        }
      }
    }
  }
}
/**
 * Fonction appelée lors des événements mouseMove sur la figure.
 * @param {event} evt  evt.id conteint l'id de la figure sur laquelle agit l'événement.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.onmousemove = function (evt) {
  const id = this.getIdDocFromEvent(evt)
  var doc = this.getDoc(id)
  if (doc === null) return
  doc.hasMouse = true
  // if (doc.isActive && doc.pointCapture != null) {
  if (doc.isActive) {
    // var th = this;
    // Ligne suivante modifiée version 4.9.9.4 car ralentit beaucoup chrome
    // doc.listePr.heapAddSetTimeOut(function(){th.mousemove(evt, doc)});
    this.mousemove(evt, id, doc)
    // Ajout version 4.9.2
    if (doc.pointCapture !== null) {
      kernel.preventDefault(evt)
      evt.stopPropagation()
    }
    // Ajout version 6.5.2
    // : Si l'utilisateur a demandé d'affecter une action supplémentaire à onmousemove
    if (doc.cbmap && doc.cbmap.has('mousemove')) doc.cbmap.get('mousemove')(evt)
  }
}
/**
 * Fonction appelée lors des événements touchMove sur la figure.
 * @param {event} evt  evt.id conteint l'id de la figure sur laquelle agit l'événement.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.ontouchmove = function (evt) {
  const id = this.getIdDocFromEvent(evt)
  var doc = this.getDoc(id)
  if (doc !== null && doc.isActive && (doc.pointCapture !== null)) {
    // var th = this;
    // Ligne suivante modifiée version 4.9.9.4 car ralentit beaucoup chrome
    // doc.listePr.heapAddSetTimeOut(function(){th.touchmove(evt)});
    this.touchmove(evt, id, doc)
    // Ajout version 6.5.2 : Si l'utilisateur a demandé d'affecter une action supplémentaire à onmousemove
    if (doc.cbmap && doc.cbmap.has('touchmove')) doc.cbmap.get('touchmove')(evt)
  }
}
/**
 * Fonction appelée lors des événements mouseDown sur la figure.
 * @param {event} evt  evt.id contient l'id de la figure sur laquelle agit l'événement.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.onmousedown = function (evt) {
  var mac
  const id = this.getIdDocFromEvent(evt)
  var svg = document.getElementById(id)
  var doc = this.getDoc(id)
  if (doc === null || !doc.isActive) return
  // Ajout version 6.5.2 : Si l'utilisateur a demandé d'affecter une action supplémentaire à onmousedown
  if (doc.cbmap && doc.cbmap.has('mousedown')) doc.cbmap.get('mousedown')(evt)
  var liste = doc.listePr
  var dimf = doc.dimf
  var couleurFond = doc.couleurFond
  var transp = doc.opacity
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // Abandonné version MtgApp pour qu'on puisse sur PC tactile utiliser mouse et touch
  if (doc.type === 'touch') return
  doc.type = 'mouse'
  if (doc.pointCapture !== null) {
    doc.pointCapture = null
    doc.listeARecalculer.retireTout()
    return
  }
  if (liste.macroEnCours !== null) {
    mac = liste.macroEnCours.macroEnCours()
    if ((mac.className === 'CMacroPause') && (mac.dureePause === 0)) {
      mac.passageMacroSuiv(svg, dimf, couleurFond, transp)
      return
    }
    if ((mac !== null) && (mac.className === 'CMacroApparition') && mac.executionPossible()) {
      mac.execute(svg, dimf, couleurFond, transp, true)
      // Attention mac ne pointe plus forcément sur la macro en cours
      if ((liste.macroEnCours !== null) && !liste.macroEnCours.macroEnCours().executionEnCours) { liste.macroEnCours.passageMacroSuiv(svg, dimf, couleurFond, transp) }
      return
    }
  }
  var info = new InfoProx()
  /*
  var tab = kernel.getMousePosition(evt)
  var par = svg.parentNode
  var posdiv = kernel.getPosition2(par)
  // var point = new Point(tab[0]-posdiv.x + par.offsetParent.scrollLeft, tab[1]-posdiv.y + par.offsetParent.scrollTop);
  var point = new Point(tab[0] - posdiv[0], tab[1] - posdiv[1])
   */
  var tab = kernel.getMousePositionToParent(evt, svg)
  var point = new Point(tab[0], tab[1])
  var nbObjetsProches = liste.procheDePourCapture(Nat.or(NatObj.NPointMobile, NatObj.NMacro),
    point, info, doc.listeExclusion, true, 'mouse')
  if (nbObjetsProches > 0) {
    if ((mac = info.infoParType[NatObj.indiceMacro].premierVoisin) !== null) {
      if (mac.executionEnCours) {
        mac.macroEnCours().termineAction(svg, dimf, couleurFond, transp)
      } else {
        if (liste.macroEnCours !== null) return
        // liste.deleteTraces(svg);
        if (mac.executionPossible()) { // Ajout version 4.8
          liste.macroEnCours = mac
          // Ajout version 5.1
          mac.setMacroLanceuse(null)
          //
          mac.initialise()
          mac.execute(svg, dimf, couleurFond, transp, true)
        }
      }
    } else {
      if (!liste.documentProprietaire.modeTraceActive) liste.deleteTraces()
      // liste.heapReset(mtg32.heapSizeWeb); // Plus utilisé verion 4.9.9.4
      // doc.pointCapture = info.premierPointVoisin; // Modifi version 4.9.2
      doc.pointCapture = info.dernierPointVoisin
      doc.listeARecalculer.retireTout()
      doc.listeARecalculer.ajouteObjetsDependantsSauf(doc.pointCapture, doc.listePr, null)
    }
  } else {
    if (liste.macroEnCours !== null) {
      if (liste.macroEnCours.className === 'CMacroSuiteMacros') {
        mac = liste.macroEnCours.macroEnCours()
        if (mac.arretParClic()) {
          mac.termineAction(svg, dimf, couleurFond, transp)
          liste.macroEnCours.passageMacroSuiv(svg, dimf, couleurFond, transp)
        }
        // passageMacroSuiv redonne éventuellement un ordre paint() suivant la nature de la macro suivante
      } else {
        // Si la macro en cours est une macro d'apparition avec clic pour objet suivant
        // on ne la désactive que si on est au dernier objet
        mac = liste.macroEnCours.macroEnCours()
        if ((mac.className === 'CMacroApparition') && mac.executionEnCours) {
          // mac.actionDansPaint(cadre);
          // cadre.paneFigure.repaint();
          this.liste.update(svg, couleurFond, transp)
        } else {
          if (mac.arretParClic()) {
            mac.termineAction(svg, dimf, couleurFond, transp)
            // this.liste.macroEnCours = null;
          }
        }
      }
    }
  }
}
/**
 * Fonction de callBack appelée lors des événements mouseMove.
 * @param {event} evt  evt.id contient l'id de la figure sur laquelle agit l'événement.
 * @param {string} id : L'id du document sur lequel agit l'événement
 * @param {CMathGraphDoc} doc : Le document sur lequel agit la figure
 * @returns {undefined}
 */
MtgAppLecteur.prototype.mousemove = function (evt, id, doc) {
  let point
  const svg = document.getElementById(id)
  // Modifié version 6.5.2
  /*
  var tab = kernel.getMousePosition(evt)
  var par = svg.parentNode
  var posdiv = kernel.getPosition2(par)
  var x = tab[0] - posdiv[0]//  + par.offsetParent.scrollLeft;
  var y = tab[1] - posdiv[1]// + par.offsetParent.scrollTop;
   */
  const tab = kernel.getMousePositionToParent(evt, svg)
  const x = tab[0]
  const y = tab[1]

  if (doc.pointCapture === null) {
    const info = new InfoProx()
    point = new Point(x, y)
    const nbObjetsProches = doc.listePr.procheDePourCapture(NatObj.NPointMobile,
      point, info, doc.listeExclusion, true, 'mouse')
    document.body.style.cursor = (nbObjetsProches > 0) ? 'pointer' : (doc.defaultCursor ? doc.defaultCursor : 'default')
  } else {
    const dimf = doc.dimf
    const couleurFond = doc.couleurFond
    const transp = doc.opacity
    if ((Math.abs(x - doc.pointCapture.x) >= 1) || (Math.abs(y - doc.pointCapture.y) >= 1)) {
      point = new Point()
      const abs = new Pointeur()
      const b = doc.pointCapture.testDeplacement(dimf, x, y, point, abs)
      if (b) {
        if (doc.pointCapture.estDeNature(NatObj.NPointCapturableNonLie)) {
          doc.pointCapture.placeEn(point.x, point.y)
        } else if (doc.pointCapture.estDeNature(NatObj.NPointLie)) {
          doc.pointCapture.donneAbscisse(abs.getValue())
        }
      }
      doc.listeARecalculer.positionne(false, dimf)
      doc.listeARecalculer.update(svg, couleurFond, transp, true)
    }
  }
}
/**
 * Fonction de callBack appelée lors des événements mouseUp.
 * @param {event} evt  evt.id contient l'id de la figure sur laquelle agit l'événement.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.ondeviceup = function (evt) {
  const id = this.getIdDocFromEvent(evt)
  const svg = document.getElementById((id))
  const doc = this.getDoc(id)
  if (doc === null || !doc.isActive) return
  if (doc.pointCapture !== null) {
    doc.pointCapture = null
    doc.listeARecalculer.retireTout()
    // Modifié version 5.0.3
    // doc.listePr.update(svg, doc.couleurFond, doc.opacity, true);
    doc.listeARecalculer.update(svg, doc.couleurFond, doc.opacity, true)
    document.body.style.cursor = 'default'
  }
}
/**
 * Fonction de callBack appelée lors des événements touchStart.
 * @param {event} evt  evt.id contient l'id de la figure sur laquelle agit l'événement.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.ontouchstart = function (evt) {
  var mac
  const id = this.getIdDocFromEvent(evt)
  var svg = document.getElementById((id))
  var doc = this.getDoc(id)
  if (doc === null || !doc.isActive) return
  // Ajout version 6.5.2 : Si l'utilisateur a demandé d'affecter une action supplémentaire à onmousedown
  if (doc.cbmap && doc.cbmap.has('touchstart')) doc.cbmap.get('touchstart')(evt)
  var liste = doc.listePr
  var dimf = doc.dimf
  var couleurFond = doc.couleurFond
  var transp = doc.opacity
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type == "mouse")) return;
  doc.type = 'touch'
  if (doc.pointCapture !== null) {
    doc.pointCapture = null
    doc.listeARecalculer.retireTout()
    return
  }
  if (liste.macroEnCours !== null) {
    mac = liste.macroEnCours.macroEnCours()
    if ((mac.className === 'CMacroPause') && (mac.dureePause === 0)) {
      mac.passageMacroSuiv(svg, dimf, couleurFond, transp)
      return
    }
    if ((mac !== null) && (mac.className === 'CMacroApparition') && mac.executionPossible()) {
      mac.execute(svg, dimf, couleurFond, transp, true)
      // Attention mac ne pointe plus forcément sur la macro en cours
      if (!liste.macroEnCours.macroEnCours().executionEnCours) liste.macroEnCours.passageMacroSuiv(svg, dimf, couleurFond, transp)
      return
    }
  }
  var info = new InfoProx()
  // Modifié version 6.5.2
  /*
  var tab = kernel.getTouchPosition(evt)
  var par = svg.parentNode
  // var offs = kernel.getOffset(par); // Version 4.9.9.4 : Ne pas appeler getOffset sur le svg mais sur le div parent
  // Modification version MtgApp car ne marchait plus avec ascenseurs sur mobile
  var offs = kernel.getPosition2(par)
  // var posdiv = kernel.getPosition2(par);
  // var point = new Point(tab[0]-posdiv.x + par.offsetParent.scrollLeft, tab[1]-posdiv.y + par.offsetParent.scrollTop);
  // var point = new Point(tab[0]-par.offsetLeft, tab[1]-par.offsetTop);
  // var point = new Point(tab[0]-offs[0], tab[1]-offs[1]);
  // Modifié version mtgApp
  var point = new Point(tab[0] - offs[0], tab[1] - offs[1])
   */
  var tab = kernel.getTouchPositionToParent(evt, svg)
  var point = new Point(tab[0], tab[1])
  var nbObjetsProches = liste.procheDePourCapture(Nat.or(NatObj.NPointMobile, NatObj.NMacro),
    point, info, doc.listeExclusion, true, 'touch')
  if (nbObjetsProches > 0) {
    if ((mac = info.infoParType[NatObj.indiceMacro].premierVoisin) !== null) {
      if (mac.executionEnCours) {
        mac.macroEnCours().termineAction(svg, dimf, couleurFond, transp)
      } else {
        if (liste.macroEnCours !== null) return
        if (!doc.modeTraceActive) liste.deleteTraces()
        liste.macroEnCours = mac
        mac.initialise()
        mac.execute(svg, dimf, couleurFond, transp, true)
      }
    } else {
      if (!doc.modeTraceActive) liste.deleteTraces()
      doc.pointCapture = info.dernierPointVoisin
      // liste.heapReset(mtg32.heapSizeMobile); // Plus utilisé version 4.9.9.4
      // var ch = evt.target.id.substring(this.liste.id.length);
      // doc.pointCapture = this.liste.get(ch);
      doc.listeARecalculer.retireTout()
      doc.listeARecalculer.ajouteObjetsDependantsSauf(doc.pointCapture, liste, null)
      kernel.preventDefault(evt)
      evt.stopPropagation()
    }
  } else {
    if (liste.macroEnCours !== null) {
      if (liste.macroEnCours.className === 'CMacroSuiteMacros') {
        mac = liste.macroEnCours.macroEnCours()
        if (mac.arretParClic()) {
          mac.termineAction(svg, dimf, couleurFond, transp)
          liste.macroEnCours.passageMacroSuiv(svg, dimf, couleurFond, transp)
        }
        // passageMacroSuiv redonne éventuellement un ordre paint() suivant la nature de la macro suivante
      } else {
        // Si la macro en cours est une macro d'apparition avec clic pour objet suivant
        // on ne la désactive que si on est au dernier objet
        mac = liste.macroEnCours.macroEnCours()
        if ((mac.className === 'CMacroApparition') && mac.executionEnCours) {
          // mac.actionDansPaint(cadre);
          // cadre.paneFigure.repaint();
          liste.update(svg, couleurFond, transp)
        } else {
          if (mac.arretParClic()) {
            mac.termineAction(svg, dimf, couleurFond, transp)
            // this.liste.macroEnCours = null;
          }
        }
      }
    }
  }
}
/**
 * Fonction de callBack appelée lors des événements touchMove.
 * @param {event} evt
 * @param {string} id : L'id du document dur lequel agit l'événement
 * @param {CMathGraphDoc} doc : Le document sur lequel agit la figure
 * @returns {undefined}
 */
MtgAppLecteur.prototype.touchmove = function (evt, id, doc) {
  var svg = document.getElementById(id)
  if (doc.pointCapture !== null) {
    var dimf = doc.dimf
    var couleurFond = doc.couleurFond
    var transp = doc.opacity
    // Modifié version 6.5.2
    /*
    var tab = kernel.getTouchPosition(evt)
    var par = svg.parentNode
    // var offs = kernel.getOffset(par); // Version 4.9.9.4 : Ne pas appeler getOffset sur le svg mais sur le div parent
    // Modification version MtgApp car ne marchait plus avec ascenseurs sur mobile
    var offs = kernel.getPosition2(par)
    // var posdiv = kernel.getPosition2(par);
    // var x = tab[0] - par.offsetLeft; // + par.offsetParent.scrollLeft;
    // var y = tab[1] - par.offsetTop; // + par.offsetParent.scrollTop;
    // var x = tab[0] - offs[0]; // + par.offsetParent.scrollLeft;
    // var y = tab[1] - offs[1]; // + par.offsetParent.scrollTop;
    // Modifié version mtgApp
    var x = tab[0] - offs[0] // + par.offsetParent.scrollLeft;
    var y = tab[1] - offs[1] // + par.offsetParent.scrollTop;
     */
    var tab = kernel.getTouchPositionToParent(evt, svg)
    var x = tab[0]
    var y = tab[1]
    if ((Math.abs(x - doc.pointCapture.x) >= 1) || (Math.abs(y - doc.pointCapture.y) >= 1)) {
      var point = new Point()
      var abs = new Pointeur()
      var b = doc.pointCapture.testDeplacement(dimf, x, y, point, abs)
      if (b) {
        if (doc.pointCapture.estDeNature(Nat.or(NatObj.NPointBase, NatObj.NPointBaseEnt,
          NatObj.NPointInterieurCercle, NatObj.NPointInterieurPolygone))) {
          doc.pointCapture.placeEn(point.x, point.y)
        } else if (doc.pointCapture.estDeNature(NatObj.NPointLie)) {
          doc.pointCapture.donneAbscisse(abs.getValue())
        }
      }
      doc.listeARecalculer.positionne(false, dimf)
      doc.listeARecalculer.update(svg, couleurFond, transp, true)
    }
  }
}
/**
 * Fonction de callBack appelée lors des événements touchEnd.
 * @param {event} evt  evt.id contient l'id de la figure sur laquelle agit l'événement.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.ontouchend = function (evt) {
  /*
  doc.type = ""; // On autorise à nouveau tous les événements
  */
  this.ondeviceup(evt)
  // Ajout version 6.5.2 : Si l'utilisateur a demandé d'affecter une action supplémentaire à onmousedown
  const id = this.getIdDocFromEvent(evt)
  var doc = this.getDoc(id)
  if (doc !== null && doc.cbmap && doc.cbmap.has('touchend')) doc.cbmap.get('touchend')(evt)
}
/**
 * Fonction de callBack appelée lors des événements touchCancel.
 * @param {event} evt  evt.id contient l'id de la figure sur laquelle agit l'événement.
 * @returns {undefined}
 */
MtgAppLecteur.prototype.ontouchcancel = function (evt) {
  this.ondeviceup(evt)
  // Ajout version 6.5.2 : Si l'utilisateur a demandé d'affecter une action supplémentaire à onmousedown
  const id = this.getIdDocFromEvent(evt)
  var doc = this.getDoc(id)
  if (doc !== null && doc.cbmap && doc.cbmap.has('touchcancel')) doc.cbmap.get('touchcancel')(evt)
}

// Quand il y a à la fois des éléments mouse et touch générés, les événements touch sont toujours en premier
// et les mouse en derniers
// On met doc.type à "" pour autoriser à nouveau tous les types d'événements sur un mouseup
MtgAppLecteur.prototype.onmouseup = function (evt) {
  this.ondeviceup(evt)
  const id = this.getIdDocFromEvent(evt)
  var doc = this.getDoc(id)
  if (doc !== null) {
    doc.type = '' // On autorise à nouveau tous les événements // Test ajouté version 6.5.2
    // Ajout version 6.5.2 : Si l'utilisateur a demandé d'affecter une action supplémentaire à onmousedown
    if (doc.cbmap && doc.cbmap.has('mouseup')) doc.cbmap.get('mouseup')(evt)
  }
}
/**
 * Renvoie un objet de type CListeObjets créé à partir d'une chaîne base64 d'une figure.
 * Sert à gérer des objets de type calcul sans gérer une figure.
 * @since 4.9.7
 * @param {String} code Le code Base64 de la figure obtenu dans mtg32 par
 *                      Edition - Copier le code de la figure.
 * @returns {CListeObjets}
 */
MtgAppLecteur.prototype.createList = function (code) {
  var doc = new CMathGraphDoc(null, false, false)
  var ba = kernel.base64Decode(code)
  var inps = new DataInputStream(ba)
  doc.read(inps)
  return doc.listePr
}

/**
 * Fonction destinée à être appelée de façon externe et remplaçant le nom
 * du point ou de la droite nommé oldName par newName, à condition qu'aucun point
 * n'ait déjà le nom newName
 * @param {string} idDoc L'id du svg contenant la figure
 * @param {string} oldName Le nom du point ou la droite à renommer
 * @param {string} newName Le nouveau nom
 */
MtgAppLecteur.prototype.rename = function (idDoc, oldName, newName) {
  var svg = document.getElementById(idDoc)
  if (svg === null) return
  if (svg.localName !== 'svg') return
  var doc = this.getDoc(idDoc)
  if (doc !== null) {
    doc.listePr.rename(oldName, newName, svg)
  }
}

/**
 * Fonction vérifiant si, dans le document d'id idDoc on peut affecter au calcul (ou à la fonction)
 * nommée calcName la formule contenue dans la chaîne de caractères formula
 * @param idDoc
 * @param calcName
 * @param formula
 * @param bSignesMultImplicit true si la formule a des signes de multiplication implicite (et donc n'utilise
 * que des variables et calculs à 1 caractère) et false sinon
 * @returns {boolean}
 */
MtgAppLecteur.prototype.syntaxValidation = function (idDoc, calcName, formula, bSignesMultImplicit = true) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) {
    var list = doc.listePr
    var ch = bSignesMultImplicit ? list.addImplicitMult(formula) : formula
    var calcul = list.pointeurParNatureCalcul(NatCal.NTtCalcouFoncRouC, calcName)
    if (calcul === null) return false
    var indDernier = list.indexOf(calcul) - 1
    var varFor = calcul.variableFormelle()
    var indErr = new Pointeur(0)
    var bcomplexe = calcul.estDeNatureCalcul(Nat.or(NatCal.NCalculComplexe,
      NatCal.NFonctionComplexe, NatCal.NFonctionComplexe2Var, NatCal.NFonctionComplexe3Var))
    return bcomplexe ? CalcC.verifieSyntaxeComplexe(list, ch, indErr,
      indDernier, varFor) : CalcR.verifieSyntaxe(list, ch, indErr,
      indDernier, varFor)
  }
  return false
}

/**
 * Fonction renvoyant une chaîne de caractères correspondant à la formule contenue dans formula mais où
 * les signes de multiplications implicites sont rajoutés en supposant que la formule ne contient
 * pas de calculs ou de fonctions avace un nom de plus de un caractère.
 * @param {string}idDoc  L'id du comument à traiter
 * @param {string} formula La formule à traiter
 * @returns {*}
 */
MtgAppLecteur.prototype.addImplicitMult = function (idDoc, formula) {
  var doc = this.getDoc(idDoc)
  if (doc !== null) {
    return doc.listePr.addImplicitMult(formula)
  }
  return formula
}
/**
 * Retourne la liste des ids des documents
 * @return {string[]}
 */
MtgAppLecteur.prototype.getDocsIds = function () {
  return this.docs.map(({ idDoc }) => idDoc)
}

/**
 * Fonction renvoyant true si chCalcul représente un calcul complexe correct syntaxiquement dans le doc d'idDoc et si ce calcul
 * représente un calcul pouvant être interprété comme un calcul vectoriel sachant que le tableau tabNames
 * contient des chaînes de caractères représentant les noms de calculs complexes pouvant être considérés comme des vecteurs
 * @param {string} idDoc
 * @param {string} chCalcul
 * @param {array} tabNames
 * @return {{syntaxOK: boolean, syntaxVecOK: boolean, isVec: boolean}}
 */
MtgAppLecteur.prototype.calcVectOK = function (idDoc, chCalcul, tabNames) {
  const doc = this.getDoc(idDoc)
  if (doc === null) return false
  const listePr = doc.listePr
  const indErr = new Pointeur(0)
  const syntaxOK = CalcC.verifieSyntaxeComplexe(listePr, chCalcul, indErr, listePr.longueur() - 1, null)
  let syntaxVecOK = false
  let isVec = false
  if (syntaxOK) {
    const calc = CalcC.ccbComp(chCalcul, listePr, 0, chCalcul.length - 1, null)
    syntaxVecOK = calc.isCalcOK4Vect(tabNames)
    isVec = calc.isCalcVect(tabNames)
  }
  return { syntaxOK: syntaxOK, syntaxVecOK: syntaxVecOK, isVec: isVec }
}

/**
 * Fonction renvoyant l'id du SVG de la figure lorsqu'une evt esr renvoyé sur un element SVG de la figure
 * ou sur le svg de la figure lui-même
 * @param {event} evt
 * @return {string}
 */
MtgAppLecteur.prototype.getIdDocFromEvent = function (evt) {
  let id
  let target = evt.target
  if (target.id === '') { // Version 6.5.2 : Si on est dans un enfant du svg element représentant l'objet graphique on remonte jusqu'au parent
    target = target.parentNode
    while (target.id.indexOf('#') === -1) target = target.parentNode
    id = target.id
    const ind = id.indexOf('#')
    id = id.substring(0, ind)
  } else {
    id = target.id
    // Les 2 lignes suivantes ajoutées version 6.5.2 car on peut avoir rajouté des écouteurs sur des éléments de la figure
    var ind = id.indexOf('#')
    if (ind !== -1) id = id.substring(0, ind)
  }
  return id
}
