/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const kernelAdd = require('../kernel/kernelAdd')
const NatCal = require('../types/NatCalAdd')
const MtgDlg = require('./MtgDlg')
const AvertDlg = require('./AvertDlg')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const MtgInput = require('./MtgInput')
const CalcR = require('../kernel/CalcR')
const CalcC = require('../kernel/CalcC')
const Pointeur = require('../types/Pointeur')
const $ = require('./../global/jQuery-ui')
module.exports = ConstImpSrcNumDlg

/**
 * Boîte de dialogue proposant une liste formé de tous les objets de type calcul
 * qui peuvent être des objets sources pour le prototype proto.
 * @constructor
 * @param app L'application MtgApp propriétaire
 * @param {CPrototype} proto
 * @param callBackOK Fonction de callBack à appeler après validation par OK.
 * Cette fonction de callBack prend un paramètre tab qui est un array de longuer le nombre
 * d'éléments sources du prototype qu'on implémente, chaque élément du tableau étant soit un pointeur
 * sur l'objet à affecter à l'élément source soit une chaine de caractères représentant un calcul valide
 * réel ou complexe, calcul qu'il faudra créer et affecter comme élément source.
 */
function ConstImpSrcNumDlg (app, proto, callBackOK) {
  // On met dans un tableau this.tab de longueur le nombre d'objets sources numériques
  // soit un entier correspondant à l'élément sélectionné dans la liste de droite soit une
  // chaine de caractères correspondant à la formule entrée par l'utilisateur
  // soit un pointeur sur l'objet sélectionné dans la liste de gauche
  MtgDlg.call(this, app, 'SrcNumDlg', callBackOK)
  this.proto = proto
  var nbSources = proto.nbSrcCal()
  this.nbSources = nbSources
  var self = this
  this.tab = []
  for (var j = 0; j < nbSources; j++) this.tab[j] = ''
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 3
  })
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('SrcNumDlg2'))
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table', {
    cellspacing: 5
  })
  tr.appendChild(tabHaut)
  // A gauche une première colonne pour contenir les numéros des éléments sources
  var td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tabHaut.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:80px'
  })
  $(caption).html(kernel.getStr('SrcNumDlg3'))
  td.appendChild(caption)
  this.selectLeft = kernel.ce('select', {
    size: 12, // Le nombre de lignes visibles par défaut
    style: 'width:80px'
  })
  td.appendChild(this.selectLeft)
  for (var i = 0; i < nbSources; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    $(option).html(i + 1)
    if (i === 0) $(option).attr('selected', 'selected')
    this.selectLeft.appendChild(option)
  }
  this.selectLeft.onchange = function () {
    self.onSelectLeftChange()
  }
  // Au milieu une première colonne pour contenir les éléments qu'on peut associer au n° choisi
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tabHaut.appendChild(td)
  caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:160px'
  })
  $(caption).html(kernel.getStr('SrcNumDlg4'))
  td.appendChild(caption)

  this.selectRight = kernel.ce('select', {
    size: 12,
    style: 'width:160px'
  })
  td.appendChild(this.selectRight)
  this.selectRight.onchange = function () {
    self.onSelectRightChange()
  }

  // A droite des deux listes un panel qui contiendra un éditeur qui pourra être caché si l'élément associé n'est pas
  // compatible avec un calcul réel ou complexe et dessous un textarea donnant des infos sur l'élément slécionné
  // dans la liste de droite
  td = kernel.ce('td')
  tabHaut.appendChild(td)
  var tabRight = kernel.ce('panel', {
    cellspacing: 1
  })
  td.appendChild(tabRight)
  tr = kernel.ce('tr')
  tabRight.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(' ' + kernel.getStr('SrcNumDlg6'))
  tr.appendChild(label)
  // Dessous un panel qui contiendra un label et au-dessous un éditeur avec les boutons Valeurs et Fonctions
  tr = kernel.ce('tr')
  tabRight.appendChild(tr)
  var tabEdit = kernel.ce('table', {
    id: 'tabedit',
    cellspacing: 3
  })
  tr.appendChild(tabEdit)
  tr = kernel.ce('tr')
  tabEdit.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('SrcNumDlg7'))
  tr.appendChild(label)
  // Ligne suivante pour l'éditeur. Elle doit commencer par un td vide car dessous il y aura des boutons Valeurs et Fonctions
  tr = kernel.ce('tr')
  tabEdit.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.input = new MtgInput({
    id: 'inputfor',
    size: 36
  })
  td.appendChild(this.input)
  // Si le input contient autre chose qu'une chaine vide, il faut désélectionner la liste de droite et cacher le panel d'info
  // sur la valeur sélectionnée
  this.input.onkeyup = function (ev) {
    this.demarquePourErreur()
    self.unselect()
    if (ev.keyCode === 13) self.tab[self.selectLeft.selectedIndex] = $('#inputfor').val()
  }
  this.input.onkeydown = function (ev) {
    if (ev.keyCode === 9) { // Tab key
      if (self.selectLeft.selectedIndex < self.selectLeft.options.length - 1) {
        self.selectLeft.selectedIndex++
        self.onSelectLeftChange()
        kernel.preventDefault(ev)
      }
    }
  }
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.input, true, app.listePr.longueur() - 1, false)
  tabEdit.appendChild(paneBoutonsValFonc)
  tr = kernel.ce('tr')
  tabRight.appendChild(tr)
  var tabInfoSel = kernel.ce('table', {
    id: 'tabinfosel',
    cellspacing: 5
  })
  tr.appendChild(tabInfoSel)
  tr = kernel.ce('tr')
  tabInfoSel.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('SrcNumDlg8'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabInfoSel.appendChild(tr)
  var inputinfosel = kernel.ce('textarea', {
    id: 'infosel',
    disabled: 'true',
    cols: 40,
    rows: 4
  })
  tr.appendChild(inputinfosel)

  // En bas un panel avec l'info sur l'élément source attendu et au-dessous un etxtarea d'infos
  // sur le prototype
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas = kernel.ce('table', {
    cellspacing: 5
  })
  tr.appendChild(tabBas)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('SrcNumDlg5') + ' ')
  div.appendChild(label)
  var inputinf = kernel.ce('input', {
    type: 'text',
    id: 'infosource',
    size: 56,
    disabled: true
  })
  div.appendChild(inputinf)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var inputinfconst = kernel.ce('textarea', {
    id: 'infoconst',
    disabled: 'true',
    cols: 75,
    rows: 6
  })
  tr.appendChild(inputinfconst)
  // On met dans infoProto la chaîne d'aide à l'exception de la partie affichée pour aider à la désignation
  // c'est à dire ce qui suit le premier # s'il y en a un
  var ch = proto.commentaire
  var k = ch.indexOf('#')
  if (k !== -1) $(inputinfconst).val(ch.substring(0, k))
  else $(inputinfconst).val(ch)
  this.oldLeftSelIndex = 0 // Pour pouvoir quand on change la sélection de gauche affecter le contenu de l'éditeur de formule
  this.initSelection()
  this.onSelectLeftChange()

  this.create('SrcNumDlg1', 650)
}

ConstImpSrcNumDlg.prototype = new MtgDlg()

/**
 * Fonction qui préselectionne dans la liste de droite les élléments qui ne sont pas des calculs éditables
 * et qui sont les seuls disponibles
 */
ConstImpSrcNumDlg.prototype.initSelection = function () {
  var i; var inf
  var app = this.app
  var list = app.listePr
  for (i = 0; i < this.nbSources; i++) {
    inf = list.listeParNatCal(app,
      kernelAdd.natObjCalPourProto(this.proto.get(i).getNatureCalcul()), list.longueur() - 1)
    if (!this.isCalc(i) && (inf.pointeurs.length === 1)) this.tab[i] = inf.pointeurs[0]
  }
}

ConstImpSrcNumDlg.prototype.onSelectLeftChange = function () {
  // On regarde si au moment où on a changé la sélection dans la liste de gauche si le conenu de l'éditeur n'était pas vide
  // Si c'est le cas on affecte son contenu à this.tab[this.oldSeelectedIndex]
  var oldind = this.oldLeftSelIndex
  var oldsel = this.tab[oldind]
  var sel = $('#inputfor')
  var str = sel.val()
  if (this.isCalc(oldind) && (typeof oldsel === 'string') && (str !== '')) this.tab[oldind] = str
  this.updateList()
  var index = this.selectLeft.selectedIndex
  $('#infosource').val(this.proto.chIndSourceNum(index))
  var editVisible = this.isCalc(index)
  $('#tabedit').css('visibility', editVisible ? 'visible' : 'hidden')
  this.oldLeftSelIndex = index
  var ch = this.tab[index]
  if (ch !== '') {
    if (typeof ch === 'string') sel.val(ch)
    else {
      sel.val('')
      this.selectRight.selectedIndex = this.inf.pointeurs.indexOf(ch)
      $('#infosel').val(ch.infoHist())
    }
  } else {
    $('#infosel').val('')
    sel.val('')
    if (!editVisible && this.inf.pointeurs.length === 1) {
      this.selectRight.selectedIndex = 0
      this.tab[index] = this.inf.pointeurs[0]
    }
  }
  // if (editVisible) sel.focus();
}

/**
 * Fonction qui désélectionne la liste de droite et vide le textarea d'info sur l'élément sélectionné
 */
ConstImpSrcNumDlg.prototype.unselect = function () {
  this.tab[this.selectLeft.selectedIndex] = ''
  this.selectRight.selectedIndex = -1
  $('#tabinfosel').css('visibility', 'hidden')
}

ConstImpSrcNumDlg.prototype.onSelectRightChange = function () {
  $('#inputfor').val('')
  $('#tabinfosel').css('visibility', 'visible')
  var el = this.inf.pointeurs[this.selectRight.selectedIndex]
  this.tab[this.selectLeft.selectedIndex] = el
  $('#infosel').val(el.infoHist())
}

ConstImpSrcNumDlg.prototype.updateList = function () {
  var app = this.app
  var list = app.listePr
  $(this.selectRight).empty()
  this.inf = list.listeParNatCal(app,
    kernelAdd.natObjCalPourProto(this.proto.get(this.selectLeft.selectedIndex).getNatureCalcul()), list.longueur() - 1)
  for (var i = 0; i < this.inf.pointeurs.length; i++) {
    var option = kernel.ce('option', {
      class: 'mtgOption'
    })
    $(option).html(this.inf.noms[i])
    this.selectRight.appendChild((option))
  }
}

ConstImpSrcNumDlg.prototype.onCancel = function () {
  this.app.activeOutilCapt()
  this.destroy()
}

ConstImpSrcNumDlg.prototype.OK = function () {
  var valide
  var app = this.app
  var list = app.listePr
  var proto = this.proto
  var nbSources = proto.nbSrcCal()
  var inderr = new Pointeur(0)
  var indmax = list.longueur() - 1
  var index = this.selectLeft.selectedIndex
  // On regarde d'abord si on était en train d'éditer une formule
  if (this.isCalc(index) && (typeof this.tab[index] === 'string')) this.tab[index] = $('#inputfor').val()

  for (var i = 0; i < nbSources; i++) {
    var ch = this.tab[i] // Peut être un pointeur ou une chaîne de caractères
    if (typeof ch === 'string') {
      // On vérifie si la syntaxe est correcte suivant que c'est un calcul réel ou complexe
      if (proto.get(i).getNatureCalcul() === NatCal.NCalculReel) { valide = CalcR.verifieSyntaxe(list, ch, inderr, indmax, null) } else { valide = CalcC.verifieSyntaxeComplexe(list, ch, inderr, indmax, null) }
      if (!valide) {
        this.selectLeft.selectedIndex = i
        this.onSelectLeftChange()
        this.input.marquePourErreur()
        new AvertDlg(app, 'Incorrect', function () {
          $('#inputfor').focus()
        })
        return
      }
    }
  }
  // Si tout est correct on appelle this.callBackOK avec comme paramètre this.tab
  this.callBackOK(this.tab)
  this.destroy()
}

/**
 * Fonction renvoyant true seulement si l'élément source numérique n° ind de this.proto
 * est un calcul réel ou complexe
 * @param ind
 */
ConstImpSrcNumDlg.prototype.isCalc = function (ind) {
  return kernelAdd.natObjCalPourProto(this.proto.get(ind).getNatureCalcul()).isOfNature(NatCal.NCalcRouC)
}
