/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const MtgInput = require('./MtgInput')
const EditeurValeurReelle = require('./EditeurValeurReelle')
const CValeur = require('../objets/CValeur')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const $ = require('./../global/jQuery-ui')
module.exports = CourbeAvecTanCoefDlg

/**
 * Boîte de dialogue pour entrer les coefficients directeurs ddes tangentes d'une courbe définie par des points (de 2 à 9)
 * @param app L'application proprétaire
 * @param tabCal Array de CCalcul qui contiennent les formules pour les coefficients directeurs (initialisés à 0)
 * @param tabPoints Array dont les éléments pointent sur les points en lesquels chaque tangenet est tracée.
 * @param indmax L'indice maximum dans la liste des objets que les formules entrées peuvent utiliser
 * @param callBackOK Fonction de callBack a apeller quand l'utilisateur clique sur OK
 * @param callBackCancel Fonction de callBack à apeller quand l'utilisateur clique sur Cancel
 * @constructor
 */
function CourbeAvecTanCoefDlg (app, tabCal, tabPoints, indmax, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'CourbeAvecTanCoeffDlg', callBackOK, callBackCancel)
  this.tabCal = tabCal
  this.indmax = indmax
  this.nbPoints = tabCal.length
  var self = this
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 2
  })
  var tr = kernel.ce('tr')
  var label = kernel.ce('label')
  var td = kernel.ce('td', {
    style: 'vertical-align:top;'
  })
  var tabLeft = kernel.ce('table')
  var tabEdit = kernel.ce('table')
  this.oldSelIndex = 0 // Pour pouvoir quand on change la sélection de gauche affecter le contenu de l'éditeur de formule s'il est correct
  this.appendChild(tabPrincipal)
  tabPrincipal.appendChild(td)
  td.appendChild(tabLeft)
  td.appendChild(tr)
  $(label).html(kernel.getStr('CourbeAvecTanDlg4'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  td.appendChild(tr)
  var select = kernel.ce('select', {
    style: 'width:80px;',
    size: this.nbPoints // Le nombre de lignes visibles par défaut
  })
  this.select = select
  td.appendChild(select)
  for (var i = 0; i < this.nbPoints; i++) {
    var option = kernel.ce('option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(tabPoints[i].nom)
    select.appendChild(option)
  }
  select.onchange = function () { self.onSelectChange() }
  td = kernel.ce('td', {
    style: 'vertical-align:top;'
  })
  tabPrincipal.appendChild(td)
  td.appendChild(tabEdit)
  tr = kernel.ce('tr')
  tabEdit.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('CoefDir') + ' :')
  td.appendChild(label)
  tr = kernel.ce('tr')
  tabEdit.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.input = new MtgInput({
    size: 16
  })
  $(this.input).val('0')
  td.appendChild(this.input)
  this.input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      if (self.validateInput(self.select.selectedIndex)) self.selectNext()
    }
  }
  this.input.onkeydown = function (ev) {
    if (ev.keyCode === 9) {
      kernel.preventDefault(ev)
      self.selectNext()
    } // Tab key
  }
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.input, true, indmax, false)
  tabEdit.appendChild(paneBoutonsValFonc)

  setTimeout(function () { self.onSelectChange() }, 500)

  // Un seul bouton Fermer pour cette boîte de dialogue
  var buttons = {}
  buttons[kernel.getStr('Term')] = function (ev) {
    self.stopEvent(ev)
    if (self.validateInput(self.select.selectedIndex)) {
      self.destroy()
      self.callBackOK()
    }
  }
  buttons[kernel.getStr('Cancel')] = function (ev) {
    self.stopEvent(ev)
    self.callBackCancel()
    self.destroy()
  }

  // Attention : Pour cette boîte de dialogue ne pas appeler create
  $('#' + self.id).dialog({
    modal: true,
    title: kernel.getStr('CourbeAvecTanDlg3'),
    buttons: buttons,
    close: function (ev) {
      self.stopEvent(ev)
      self.destroy()
    },
    width: 280,
    closeOnEscape: false,
    // On centre la boîte de dialogue sur le div parent de l'application
    position: { my: 'right top', at: 'right top', of: '#divMtgDlg' }
  })
}

CourbeAvecTanCoefDlg.prototype = new MtgDlg()

CourbeAvecTanCoefDlg.prototype.validateInput = function (index) {
  var app = this.app
  var list = app.listePr
  var valeur = new CValeur(list) // Sert à éditer les formules
  var editeur = new EditeurValeurReelle(app, this.input, this.indmax, valeur, null)
  if (editeur.validate($(this.input).val())) {
    var calc = this.tabCal[index]
    calc.calcul = valeur.calcul
    calc.chaineCalcul = valeur.calcul.chaineCalculSansPar(null)
    list.positionneDependantsDe(false, app.dimf, calc)
    list.updateDependants(calc, app.svgFigure, app.doc.couleurFond, app.doc.opacity, true)
    return true
  } else {
    this.input.focus()
    return false
  }
}

CourbeAvecTanCoefDlg.prototype.selectNext = function () {
  this.oldSelIndex = this.select.selectedIndex
  this.select.selectedIndex = (this.select.selectedIndex + 1) % this.nbPoints
  this.onSelectChange()
}

CourbeAvecTanCoefDlg.prototype.onSelectChange = function () {
  if (this.validateInput(this.oldSelIndex)) {
    this.oldSelIndex = this.select.selectedIndex
    $(this.input).val(this.tabCal[this.oldSelIndex].chaineCalcul)
  } else this.select.selectedIndex = this.oldSelIndex
  $(this.input).focus()
  $(this.input).select()
}
