/*!
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 * @version 5.7.0
 */

// Avant @babel/polyfill s'occupait d'ajouter les polyfill pour la conf babel useBuiltIns
// Depuis babel 7.4 fait préciser la version de core-js et inclure directement core-js
// cf https://www.thebasement.be/updating-to-babel-7.4/
// https://babeljs.io/blog/2019/03/19/7.4.0

// Avec du useBuiltIns entry|usage faut faire une seule fois l'import de core-js,
// on le fait dans ce fichier qui devient l'unique point d'entrée
require('core-js/stable')
// celui-là ne serait nécessaire que si on utilisait des générateurs ou du async/await
// require('regenerator-runtime/runtime')

// ATTENTION, avec useBuiltIns usage, faut préciser dans la conf babel
// "sourceType": "unambiguous",
// sinon ça plante en disant qu'on peut pas affecter module.exports

const loadMathJax = require('./loadMathJax')

/**
 * Fonction de chargement d'une figure mathgraph dans un DOM
 */
module.exports = mtgLoad

// notre instance unique du player
let appPlayerPromise

/**
 * Retourne un id non existant dans le dom (de la forme svgMtgXX où on incrémente XX jusqu'à en trouver un inutilisé)
 * @private
 * @return {string}
 */
function getSvgNewId () {
  let i = 0
  while (document.getElementById(`svgMtg${i}`)) i++
  return `svgMtg${i}`
}
/**
 * Retourne le svg, créé s'il n'existait pas, avec les bonnes options
 * @private
 * @param {HTMLElement} container
 * @param {Object} [svgOptions]
 * @param {string} [svgOptions.idSvg=svgMtg]
 * @param {number} [svgOptions.height=650]
 * @param {number} [svgOptions.width=900]
 * @return {HTMLElement} le svg
 */
function getSvg (container, svgOptions) {
  if (!svgOptions) svgOptions = {}
  const width = svgOptions.width || 900
  const height = svgOptions.height || 650
  const idSvg = svgOptions.idSvg || getSvgNewId()
  let svg = document.getElementById(idSvg)
  if (svg === null) {
    svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg')
    svg.setAttribute('id', idSvg)
    container.appendChild(svg)
  }
  svg.setAttribute('width', width)
  svg.setAttribute('height', height)
  // Ajouté par Yves version 6.6.1 pour même fonctionnement dans le player que dans l'application
  svg.setAttribute('shape-rendering', 'geometricPrecision')
  return svg
}

/**
 * Initialise le container avec un svg, charge mathJax si besoin et le configure
 * @private
 * @param {HTMLElement} container
 * @param {MtgOptions} mtgOptions
 * @return {Promise}
 */
function init (container, mtgOptions) {
  // on vire les navigateurs sans Promise (IE)
  if (typeof Promise === 'undefined' || typeof Array.from !== 'function') {
    return Promise.reject(Error('Désolé, votre navigateur est trop ancien et ne peut pas afficher de figures Mathgraph.'))
  }
  // pour coreOnly, pas besoin de container ni de svg
  if (mtgOptions.loadCoreOnly === true) return Promise.resolve()
  // idem avec MathJax
  const mjOptions = {}
  if (mtgOptions.electron) mjOptions.mathjax3Base = 'MathJax3'
  if (mtgOptions.loadCoreWithMathJax === true) return loadMathJax(mjOptions)
  // pour le reste faut un conteneur
  if (!container) return Promise.reject(Error('Il faut passer un conteneur'))
  if (typeof container.appendChild !== 'function') return Promise.reject(Error('Conteneur invalide'))
  // conteneur qu'on va initialiser
  while (container.firstChild) container.removeChild(container.firstChild)
  container.appendChild(document.createTextNode('Chargement en cours…'))
  // Pour que MathJax ne traite pas le contenu
  container.setAttribute('class', 'text2jax_ignore')
  // pour ne pas avoir ce bug https://github.com/mathjax/MathJax/issues/2257, on colle le font-family de body à notre container
  try {
    const rootFontFamily = getComputedStyle(document.body).fontFamily
    console.log(`body a la police ${rootFontFamily} que l’on impose à notre container pour éviter un bug MathJax`)
    if (rootFontFamily) container.setAttribute('style', `font-family: ${rootFontFamily}`)
  } catch (error) {
    console.error('plantage lors de la récup de body.style.fontFamily', error)
  }

  return loadMathJax(mjOptions)
}

/**
 * L'objet à passer à la place du code base64 d'une figure pour initialiser une figure vide
 * @typedef NewFigOptions
 * @property {string} [type=unity] simple|unity|orthonormal|orthogonal
 *    simple pour une figure sans longueur unité sans repère
 *    unity pour une figure avec segment longueur unité et sans repère
 *    orthonormal pour une figure avec repère orthonormal
 *    orthogonal pour un figure avec repère orthogonal
 * @property {object} datarep objet pour initialiser le repère
 * @property {boolean} datarep.quadver true si on veut que le repère soit quadrillé verticalement
 * @property {boolean} datarep.quadhor true si on veut que le repère soit quadrillé horizontalement
 * @property {boolean} datarep.withvect true si le repère doit avoir des vecteurs directeurs sur chaque axe
 * @property {boolean} datarep.grid true si on veut que le repère ait des pointillés aux coordonnées entières
 * @property {string} datarep.grad no|simple|trig, avec
 *                                no si on ne veut pas de graduations sur les axes
 *                                simple si on veut des graduations usuelles
 *                                trig si on veut des graduations trigo sur l'axe des abscisses
 *                                  (dans ce cas le paramètre unity de datfig n'est pas pris en
 *                                   compte et l'unité de la figure est le radian)
 */
/**
 * Callback functionOnSave (passée via MtgOptions)
 * @callback functionOnSave
 * @param {object} result
 * @param {string} result.fig La figure en base64
 * @param {number|undefined} result.score Un éventuel score (0 ou 1, pour les exercices de construction)
 */

/**
 * Options pour initialiser l'application
 * @typedef MtgOptions
 * @property {string} [fig] La figure base64 à afficher au chargement
 * @property {number|string} level entier de 0 à 3 pour le niveau de fonctionnement du logiciel
 *                                              (ou une chaine de caractères Base64 donnant
 *                                              les outils permis ou interdits,
 *                                              mais cette option est plutôt à usage interne)
 *                                            0 : sélection d'outils pour le niveau élémentaire
 *                                            1 : collège
 *                                            2 : lycée (sans nombres complexes)
 *                                            3 : lycée avec nombres complexes
 * @property {boolean|string} [local=false] Si l'application fonctionne en local hors-ligne,
 *                            passer true ou une chaîne de caractères donnant le chemin relatif de
 *                            mtgLoad.js par rapport à la page qui a lancé l'application
 * @property {boolean} [displayOnLoad=true] Passer false pour ne pas afficher la figure
 *                            automatiquement dès qu'elle est prête (il faudra utiliser les méthodes
 *                            du player pour l'afficher plus tard)
 * @property {boolean} [displayMeasures=true] Passer false pour ne pas afficher automatiquement les mesures de longueur et d'angles
 * @property {boolean} [isInteractive=true] Passer false pour ne pas permettre de bouger les points (player only)
 * @property {boolean} [isEditable=true] Passer false pour afficher la figure avec le player (ni menu ni bouton)
 * @property {boolean} [loadCoreOnly=false] Passer true pour ne charger que le core (ça retourne un mtgAppLecteur instancié sans figure, pas de svg créé ni de MathJax chargé)
 * @property {boolean} [loadCoreWithMathJax=false] Passer true pour ne charger que le core et MathJax (ça retourne un mtgAppLecteur instancié sans figure ni de svg créé), pour faire ensuite des addDoc dans des svg que l'appelant devra créer
 * @property {boolean} [pointsAuto=true] Passer false pour ne pas créer automatiquement un point quand on clique sur un endroit vide (avec certains outils)
 * @property {boolean} [randomOnInit] passer true pour lancer l'aléatoire à l'initialisation de la figure (lecteur seulement)
 * @property {boolean} [stylePointCroix=false] Passer true pour que le style de point au démarrage soit la grande croix.
 * @property {boolean} [codeBase64=true] Passer false retirer l'outil permettant d'obtenir le code Base64 de la figure
 * @property {boolean} [open=true] Passer false pour retirer l'outil permettant d'ouvrir une figure
 * @property {boolean} [save=true] Passer false pour retirer l'outil permettant de sauvegarder une figure
 * @property {boolean} [newFig=true] Passer false pour retirer l'outil permettant de créer une nouvelle figure
 * @property {boolean} [options=true] Passer false pour retirer l'outil permettant de changer les options de la figure (ajouter ou retirer des outils)
 * @property {boolean} [onlyPoints=false] Passer true pour que les outils de transformation ne s'appliquent qu'à des points
 * @property {boolean} [dys=false] Passer true pour démarrer l'interface avec un affichage adapté aux utilisateurs "dys" (traits épais, points en grande croix, taille des noms plus grande)
 * @property {number} [zoomFactor=1] Facteur d'agrandissement ou de réduction des icones
 * @property {functionOnSave} [functionOnSave] callBack appelée au clic sur le bouton d'enregistrement (sera appelée avec {figure, result}, pour le moment sans arguments)
 * @property {function} [callBackAfterReady] callBack appelée lorsque la figure est chargée, les méthodes de MtgApp peuvent alors être utilisées (inutile pour player et coreOnly)
 * @property {boolean} [bplayer=false] Passer true pour ajouter une propriété player (de type MtgAppLecteur) à l'objet MtgApp retourné (utile pour les exercices de construction)
 * @property {string} [language=''] Passer fr|en|es pour forcer la langue (sinon détection automatique d'après les headers envoyés par le navigateur)
 * @property {boolean} [electron=false] Passer true si destiné à fonctionner avec la version electron
 * @property {boolean} [edition=false] Si on édite un exerice de construction il faut que ce paramètre soit true
 */

/**
 * @callback mtgLoadCallback
 * @param {Error} [error]
 * @param {MtgApp|MtgAppLecteur} mtgApp L'appli instanciée (mais le rendu de la figure peut être en cours)
 */

/**
 * Sert à inclure toutes les dépendances de l'éditeur mathgraph
 * on passe par un js externe pour que webpack compile tout ça dans un js séparé
 * qui ne sera pas chargé dans le cas player
 * @private
 * @param {boolean} isElectron
 * @return {Promise<Array>} Le 1er élément de l'array est MtgApp
 */
function fetchMtgApp (isElectron) {
  // pour tout charger en parallèle faut passer par du Promise.all
  // car import() prend une toujours une string (gère pas un Array)
  if (isElectron) {
    return Promise.all([
      import('./MtgApp'),
      import('./mtgLoad.dependenciesForElectron')
    ])
  }
  return Promise.all([
    import('./MtgApp'),
    import('./mtgLoad.dependencies')
  ])
}

/**
 * Chargement player (pour instancier le player une seule fois)
 * @private
 * @return {Promise<MtgAppLecteur>}
 */
function fetchMtgAppLecteurInstance () {
  // pour le code splitting (https://webpack.js.org/api/module-methods/#require-ensure)
  // on passera à import() quand on fera une màj webpack
  if (!appPlayerPromise) {
    appPlayerPromise = import('./MtgAppLecteur').then(function ({ default: MtgAppLecteur }) {
      if (!MtgAppLecteur) return Promise.reject(Error('Impossible de charger le lecteur Mathgraph'))
      return new MtgAppLecteur()
    })
  }
  return appPlayerPromise
}

/**
 * Chargement core only (pas de figure donc pas de svg)
 * @private
 * @param mtgOptions (on utilise que callBackAfterReady)
 * @return {Promise<MtgAppLecteur>}
 */
function loadMtgCore (mtgOptions) {
  return fetchMtgAppLecteurInstance(mtgOptions).then((mtgAppLecteur) => {
    // console.log('instance MtgAppLecteur', Object.keys(mtgAppLecteur.prototype).join(' '))
    // si y'a un mtgOptions.callBackAfterReady, on l'appelle, mais après l'appel de la cb de chargement
    // c'est complètement inutile mais là juste pour l'homogénéité
    if (typeof mtgOptions.callBackAfterReady === 'function') setTimeout(() => mtgOptions.callBackAfterReady(mtgAppLecteur), 0)
    return mtgAppLecteur
  })
}

/**
 * Retourne une promesse résolue avec MtgApp (mis dans le container et prêt à servir)
 * @private
 * @param {string|HTMLElement} container
 * @param {Object} svgOptions
 * @param {Object} mtgOptions
 * @return {Promise<Array>}
 */
function loadMtgEditor (container, svgOptions, mtgOptions) {
  const isElectron = mtgOptions.electron === true
  return fetchMtgApp(isElectron).then(function ([{ default: MtgApp }]) {
    if (!MtgApp) return Promise.reject(Error('Impossible de charger l’application Mathgraph'))
    // on vide le texte "Chargement en cours…" mis par init
    while (container.firstChild) container.removeChild(container.firstChild)
    const svg = getSvg(container, svgOptions)
    return new MtgApp(svg, mtgOptions)
  })
}

/**
 * Retourne une promesse résolue avec le player
 * @private
 * @param container
 * @param svgOptions
 * @param mtgOptions
 * @return {Promise<MtgAppLecteur>}
 */
function loadMtgPlayer (container, svgOptions, mtgOptions) {
  if (!mtgOptions.fig) return Promise.reject(Error('Figure absente, impossible de l’afficher'))
  return fetchMtgAppLecteurInstance(mtgOptions).then((mtgAppLecteur) => {
    // on vide le texte "Chargement en cours…" mis par init
    while (container.firstChild) container.removeChild(container.firstChild)
    const svg = getSvg(container, svgOptions)
    // true par défaut
    if (mtgOptions.isInteractive !== false) mtgOptions.isInteractive = true
    mtgAppLecteur.addDoc(svg.id, mtgOptions.fig, true /* displayOnLoad */, mtgOptions.isInteractive)
    mtgAppLecteur.calculateFirstTime(svg.id, Boolean(mtgOptions.randomOnInit))
    if (mtgOptions.displayOnLoad !== false) mtgAppLecteur.display(svg.id)
    // si y'a un mtgOptions.callBackAfterReady, on l'appelle, mais après l'appel de la cb de chargement
    if (typeof mtgOptions.callBackAfterReady === 'function') setTimeout(() => mtgOptions.callBackAfterReady(mtgAppLecteur), 0)
    return mtgAppLecteur
  })
}

/**
 * Retourne la promesse de chargement
 * @private
 * @param container
 * @param svgOptions
 * @param mtgOptions
 * @return {Promise<MtgApp|MtgAppLecteur>}
 */
function mtgLoadPromise (container, svgOptions, mtgOptions) {
  if (container && typeof container === 'string') container = document.getElementById(container)
  return init(container, mtgOptions).then(() => {
    if (mtgOptions.loadCoreOnly === true || mtgOptions.loadCoreWithMathJax === true) return loadMtgCore(mtgOptions)
    if (mtgOptions.isEditable === false) return loadMtgPlayer(container, svgOptions, mtgOptions)
    return loadMtgEditor(container, svgOptions, mtgOptions)
  })
}

// cf https://jsdoc.app/about-tutorials.html et https://jsdoc.app/about-configuring-jsdoc.html
/**
 * Fonction chargeant l'application MtgApp ou MtgAppLecteur (suivant mtgOptions.isEditable)
 * Renverra l'instance de l'application à la callback ou dans une promesse
 * Voir les tutoriels {@tutorial loadEditor}, {@tutorial loadPlayer}, {@tutorial loadCore} pour quelques exemples
 * @param {HTMLElement|string} container Le conteneur pour y mettre le SVG (interface de l'appli), ou son id
 * @param {object} svgOptions Pour le svg à créer dans le dom
 * @param {number} [svgOptions.width=900] Largeur du SVG contenant l'appli
 * @param {number} [svgOptions.height=650] Hauteur du SVG contenant l'appli
 * @param {string} [svgOptions.idSvg=svgMtg] id html du svg à créer (ou à prendre dans le DOM s'il y est déjà)
 * @param {MtgOptions} mtgOptions Les informations sur l'initialisation de l'application
 * @param {mtgLoadCallback} [cb] Si non fournie mtgLoad retournera une promesse (résolue avec l'instance de l'appli)
 * @return {undefined|Promise<MtgApp>}
 */
function mtgLoad (container, svgOptions, mtgOptions, cb) {
  const promise = mtgLoadPromise(container, svgOptions, mtgOptions)
  if (!cb) return promise
  if (typeof cb !== 'function') throw Error('Si un 4e argument est passé à mtgLoad ça doit être une fonction')
  // syntaxe avec cb
  promise.then((app) => {
    cb(null, app)
  }).catch(cb)
}
