/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatArc = require('../types/NatArc')
const Vect = require('../types/Vect')
const kernel = require('../kernel/kernel')
const CArcDeCercle = require('./CArcDeCercle')
module.exports = CGrandArcDeCercle

/**
 * Classe représentant un grand arc de cercle défini par son centre, le point de départ et soit
 * un point donnant la dircetion de fin soit la valeur de l'angle au centre.
 * @constructor
 * @extends CArcDeCercle
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire.
 * @param {boolean} estElementFinal  true si élément final de construction.
 * @param {Color} couleur
 * @param {Boollan} masque  true si l'objet est masqué.
 * @param {StyleTrait} style  le style de tracé.
 * @param {CPt} o  Le centre de l'arc.
 * @param {CPt} a  Le point donnant le début de l'arc.
 * @param {CPt} pointFinArc  Le point donnant la fin de l'arc.
 * Si null c'est que l'arc est défini par son angle au centre.
 * @param {CValeurAngle} angleAuCentre  L'angle au centre (si pointFinArc est null)
 * @returns {CGrandArcDeCercle}
 */
function CGrandArcDeCercle (listeProprietaire, impProto, estElementFinal, couleur, masque,
  style, o, a, pointFinArc, angleAuCentre) {
  if (arguments.length === 1) CArcDeCercle.call(this, listeProprietaire)
  else {
    CArcDeCercle.call(this, listeProprietaire, impProto, estElementFinal, couleur,
      masque, style)
    this.o = o
    this.a = a
    this.pointFinArc = pointFinArc
    this.angleAuCentre = angleAuCentre
  }
}
CGrandArcDeCercle.prototype = new CArcDeCercle()
CGrandArcDeCercle.prototype.constructor = CGrandArcDeCercle
CGrandArcDeCercle.prototype.superClass = 'CArcDeCercle'
CGrandArcDeCercle.prototype.className = 'CGrandArcDeCercle'

CGrandArcDeCercle.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.o)
  var ind2 = listeSource.indexOf(this.a)
  var ptFinArc = null
  var angleAuCentreClone = null
  if (this.pointFinArc === null) { angleAuCentreClone = this.angleAuCentre.getClone(listeSource, listeCible) } else {
    var ind3 = listeSource.indexOf(this.pointFinArc)
    ptFinArc = listeCible.get(ind3, 'CPt')
    angleAuCentreClone = null
  }
  var ind4 = listeSource.indexOf(this.impProto)
  return new CGrandArcDeCercle(listeCible, listeCible.get(ind4, 'CImplementationProto'),
    this.estElementFinal, this.couleur, this.masque, this.style.getClone(), listeCible.get(ind1, 'CPt'),
    listeCible.get(ind2, 'CPt'), ptFinArc, angleAuCentreClone)
}
CGrandArcDeCercle.prototype.path = function () {
  var d
  var u = new Vect(this.centreX, this.centreY, this.origine_x, this.origine_y)
  var v = new Vect()
  u.tourne(this.valeurAngleAuCentre, v)
  var xf = this.centreX + v.x
  var yf = this.centreY + v.y
  var sweepflag = (this.valeurAngleAuCentre < 0) ? '0 ' : '1 '
  d = 'M ' + this.origine_x.toString() + ' ' + this.origine_y + 'A' + this.rayon + ',' +
    this.rayon + ' 0 1 ' + sweepflag + xf.toString() + ',' + yf.toString()
  return d
}
CGrandArcDeCercle.prototype.surArc = function (xt, yt) {
  var ang, ang1aux, ang2aux, dif, d

  var w = new Vect(this.centreX, this.centreY, xt, yt)
  ang = w.angleRad()
  ang1aux = this.ang1
  ang2aux = this.ang2
  if (ang1aux > ang2aux) {
    d = ang2aux
    ang2aux = ang1aux
    ang1aux = d
  }
  dif = ang2aux - ang1aux
  if (kernel.zeroAngle(dif - Math.PI)) {
    if (this.ang1 >= this.ang2) return ((ang >= this.ang2) && (ang <= this.ang1))
    else {
      return ((ang >= this.ang2) || (ang <= this.ang1))
    }
  } else {
    if (dif < Math.PI) {
      return ((ang <= ang1aux) || (ang >= ang2aux))
    } else {
      return ((ang >= ang1aux) && (ang <= ang2aux))
    }
  }
}
CGrandArcDeCercle.prototype.abscisseCurviligne = function (ang) {
  var difang
  if (this.valeurAngleAuCentre === 0) return 0
  if (this.valeurAngleAuCentre >= 0) {
    if (ang <= this.ang1) difang = this.ang1 - ang
    else difang = 2 * Math.PI - ang + this.ang1
  } else {
    if (ang >= this.ang1) difang = ang - this.ang1
    else difang = 2 * Math.PI - this.ang1 + ang
  }
  return difang / (2 * Math.PI - Math.abs(this.valeurAngleAuCentre))
}
CGrandArcDeCercle.prototype.natureArc = function () {
  return NatArc.GrandArc
}
