/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Vect = require('../types/Vect')
const CTransformation = require('./CTransformation')
const CHomothetie = require('./CHomothetie')
module.exports = CInversion

/**
 * Classe représentant l'inversion (transformation).
 * Cette transformation est spéciale car ne peut être appliquée qu'à un point.
 * @constructor
 * @extends CHomothetie
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire.
 * @param {boolean} estElementFinal  true si élément final de construction.
 * @param {CPt} centre  Le centre de la similitude.
 * @param {CValeur} rapport  Le rapport de l'inversion
 * @returns {CInversion}
 */
function CInversion (listeProprietaire, impProto, estElementFinal, centre, rapport) {
  if (arguments.length === 1) CHomothetie.call(this, listeProprietaire)
  else {
    CHomothetie.call(this, listeProprietaire, impProto, estElementFinal, centre, rapport)
  }
}
CInversion.prototype = new CHomothetie()
CInversion.prototype.constructor = CInversion
CInversion.prototype.superClass = 'CHomothetie'
CInversion.prototype.className = 'CInversion'

/**
 * Indique la nature de la transformation
 * @returns {CTransformation.inversion}
 */
CInversion.prototype.natureTransformation = function () {
  return CTransformation.inversion
}
CInversion.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.centre)
  var ind2 = listeSource.indexOf(this.impProto)
  var rapportClone = this.rapport.getClone(listeSource, listeCible)
  return new CInversion(listeCible, listeCible.get(ind2, 'CImplementationProto'),
    this.estElementFinal, listeCible.get(ind1, 'CPt'), rapportClone)
}
CInversion.prototype.depDe = function (p) {
  if (this.elementTestePourDependDe === p) return this.dependDeElementTeste
  var resultat = CHomothetie.prototype.depDe.call(this, p)
  if (this.listeProprietaire.pointeurLongueurUnite === null) { return this.memDep(resultat) } else return this.memDep(resultat || (this.listeProprietaire.pointeurLongueurUnite.depDe(p)))
}
CInversion.prototype.dependDePourBoucle = function (p) {
  var resultat = CHomothetie.prototype.depDe.call(this, p)
  if (this.listeProprietaire.pointeurLongueurUnite === null) return resultat
  else return resultat || (this.listeProprietaire.pointeurLongueurUnite.dependDePourBoucle(p))
}
CInversion.prototype.positionne = function (infoRandom) {
  var pLongUnite, longueurUnite
  CHomothetie.prototype.positionne.call(this, infoRandom)
  // A noter : valeurRapport a été calculé dans Homothetie.positionne()
  pLongUnite = this.listeProprietaire.pointeurLongueurUnite
  this.existe = this.existe && pLongUnite.existe
  if (!this.existe) return
  longueurUnite = pLongUnite.rendLongueur()
  if (longueurUnite === 0) this.existe = false
}
// Ajout version 6.3.0
CInversion.prototype.positionneFull = function (infoRandom) {
  this.rapport.dejaPositionne = false
  this.positionne(infoRandom)
}
/**
 * Renvoie dans pointr.x et pointr.y les coordonnées du point image du point de coordonnées (x,y).
 * @param {number} x
 * @param {number} y
 * @param {Object} pointr
 * @returns {undefined}
 */
CInversion.prototype.transformePoint = function (x, y, pointr) {
// Le point image du point d'affixe z est le point d'affixe z'
// défini par z' - w = k(z - w)/|z - w|², mais le module de
// z - w doit être calculé relativement à la longueur unité
// (w désigant l'affixe du centre)
  var pLongUnite, longueurUnite

  pLongUnite = this.listeProprietaire.pointeurLongueurUnite
  longueurUnite = pLongUnite.rendLongueur()

  var u = new Vect(this.centre.x, this.centre.y, x, y)
  var longueur = u.norme() / longueurUnite
  var d = longueur * longueur
  pointr.x = this.centre.x + this.valeurRapport * u.x / d
  pointr.y = this.centre.y + this.valeurRapport * u.y / d
}
CInversion.prototype.pointAUneImage = function (point) {
  return (!point.egal(this.centre))
}
