/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CMacro = require('./CMacro')
const CMacroAvecListe = require('./CMacroAvecListe')
const CSousListeObjets = require('./CSousListeObjets')
module.exports = CMacroClignotement

/**
 * Macro faisant clignoter des objets sur la figure.
 * @constructor
 * @extends CMacroAvecListe
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire.
 * @param {boolean} estElementFinal  true si l'objet est un élément final de construction.
 * @param {Color} couleur  La couleur d'éciture de l'éditeur (et du cadre éventuel).
 * @param {number} xNom  L'abscisse d'affichage de la macro
 * @param {number} yNom  L'ordonnée d'affichage de la macro
 * @param {number} decX  Décalage horizontal du nom
 * @param {number} decY  Décalage vertical du nom
 * @param {boolean} masque  true si l'éditeur est masqué
 * @param {CPt} pointLie  null ou pointe sur un point auquel l'affichage est lié.
 * @param {number} taillePolice  Indice de la taiile de police utilisée
 * @param {boolean} effacementFond  true si on efface le fond de l'en-tête.
 * @param {Color} couleurFond  La couleur de fond de l'en-tête.
 * @param {number} alignementHorizontal  0 pour alignement gauche, 1 pour centre, 2 pour droite.
 * @param {number} alignementVertical  0 pour alignement vers le haut, 1 pour centré, 2 pour bas.
 * @param {String} intitule  Le titre de la macro
 * @param {String} commentaireMacro  Eventuel commentaire explicatif.
 * @param {CListeObjets} listeAssociee  La liste des objets qui doivent clignoter
 * @param {number} dureeAnimation ; La durée de clignotement en secondes, 0 pour un clignotement
 * jusqu'au prochain clic de souris.
 * @returns {CMacroClignotement}
 */
function CMacroClignotement (listeProprietaire, impProto, estElementFinal, couleur,
  xNom, yNom, decX, decY, masque, pointLie, taillePolice, effacementFond, couleurFond, alignementHorizontal,
  alignementVertical, intitule, commentaireMacro, listeAssociee, dureeAnimation) {
  if (arguments.length !== 0) {
    if (arguments.length === 1) CMacroAvecListe.call(this, listeProprietaire)
    else {
      CMacroAvecListe.call(this, listeProprietaire, impProto, estElementFinal, couleur,
        xNom, yNom, decX, decY, masque, pointLie, taillePolice, effacementFond, couleurFond,
        alignementHorizontal, alignementVertical, intitule, commentaireMacro, listeAssociee)
      this.dureeAnimation = dureeAnimation
    }
  }
}
CMacroClignotement.prototype = new CMacroAvecListe()
CMacroClignotement.prototype.constructor = CMacroClignotement
CMacroClignotement.prototype.superClass = 'CMacroAvecListe'
CMacroClignotement.prototype.className = 'CMacroClignotement'

CMacroClignotement.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.impProto)
  var listeAssocieeClone = new CSousListeObjets()
  listeAssocieeClone.setImage(this.listeAssociee, listeSource, listeCible)
  return new CMacroClignotement(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.couleur, this.xNom, this.yNom, this.decX, this.decY,
    this.masque, this.pointLie, this.taillePolice, this.effacementFond, this.couleurFond,
    this.alignementHorizontal, this.alignementVertical, this.intitule, this.commentaireMacro,
    listeAssocieeClone, this.dureeAnimation)
}
CMacroClignotement.prototype.typeAnimation = function () {
  return CMacro.animationParTimer
}
CMacroClignotement.prototype.arretParClic = function () {
  return (this.dureeAnimation === 0)
}
CMacroClignotement.prototype.termineAction = function (svg, dimf, couleurFond, transparence) {
  clearInterval(this.timer)
  this.executionEnCours = false
  this.listeAssociee.montreTout(true)
  // this.listeAssociee.updateComposants(svg, couleurFond, transparence);
  this.listeAssociee.update(svg, couleurFond, transparence, true)
  this.passageMacroSuiv(svg, dimf, couleurFond, transparence)
}
CMacroClignotement.prototype.execute = function (svg, dimf, couleurFond, transparence) {
  var timeDebut = +new Date()
  if (this.dureeAnimation !== 0) this.timeFin = timeDebut + this.dureeAnimation * 100 // La durée est en dixièmes de secondes
  this.executionEnCours = true
  this.nombrePairClignotements = true
  var t = this
  this.timer = setInterval(function () {
    CMacroClignotement.executePourTimer.call(t, svg, dimf, couleurFond,
      transparence)
  }, 500)
}
/**
 * Fonction de callBack appelée par un timer dans execute()
 * @param {svg} svg  Le svg de la figure.
 * @param {Dimf} dimf  Les dimensions du svg
 * @param {Color} couleurFond  La couleur de fond de la figure.
 * @param {number} transparence  Le coeficient de transparence de la figure.
 * @returns {undefined}
 */
CMacroClignotement.executePourTimer = function (svg, dimf, couleurFond, transparence) {
  if (!this.executionEnCours) return
  this.nombrePairClignotements = !this.nombrePairClignotements
  this.listeAssociee.montreTout(this.nombrePairClignotements)
  this.listeAssociee.update(svg, couleurFond, transparence, true)
  var time = +new Date() // Raccourci pour (new Date()).getTime()
  if (time > this.timeFin) this.termineAction(svg, dimf, couleurFond, transparence)
}
CMacroClignotement.prototype.ajouteAntecedents = function (liste) {
  liste.ajouteElementsDe(this.listeAssociee)
}
CMacroClignotement.prototype.read = function (inps, list) {
  CMacroAvecListe.prototype.read.call(this, inps, list)
  this.dureeAnimation = inps.readInt()
}
CMacroClignotement.prototype.write = function (oups, list) {
  CMacroAvecListe.prototype.write.call(this, oups, list)
  oups.writeInt(this.dureeAnimation)
}
