// Ajout le 3/5/2021 par Yves Biton
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
import mathjs from '../kernel/mathjs'
const CCalculAncetre = require('./CCalculAncetre')
const CValeur = require('./CValeur')
const kernel = require('../kernel/kernel')
const NatCal = require('../types/NatCal')
// const matrix = require('matrix-js')
// module.exports = CMatriceAleat
export default CMatriceAleat

/**
 * Classe représentant une matrice de n lignes et p colonnes à valeurs entières distinctes
 * comprises entre les valeurs rendues par min et max
 * Les valeurs générées pour la matrice ne pourront être distinctes quue si max-min+1 est
 * supérieur ou égal au nombre de termes de la matrice
 * @param {CListeObjets} listeProprietaire : La liste propriétaire
 * @param {CImplementationProto} impProto  null ou a construction propriétaire.
 * @param {boolean} estElementFinal  true si élément final de construction
 * @param {String} nomCalcul  Le nom donné au calcul.
 * @param {int} n Le nombre de lignes de la matrice
 * @param {int} p Le nombre de colonnes de la matrice
 * @param {CValeur} min La valeur mini des termes de la matrice (dynamique)
 * @param {CValeur} max La valeur maxi des termes de la matrice (dynamique)
 * @constructor
 */
function CMatriceAleat (listeProprietaire, impProto, estElementFinal, nomCalcul, n, p, min, max) {
  if (arguments.length === 0) CCalculAncetre.call(this)
  else {
    if (arguments.length === 1) CCalculAncetre.call(this, listeProprietaire)
    else {
      CCalculAncetre.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul)
      this.n = n // Le nombre de lignes
      this.p = p // Le nombre de colonnes
      this.min = min // Valeur mini entière pour les termes de la matrice
      this.max = max // Valeur maxi entière pour les termes de la matrice
    }
  }
}

CMatriceAleat.prototype = new CCalculAncetre()
CMatriceAleat.prototype.constructor = CMatriceAleat
CMatriceAleat.prototype.superClass = 'CCalculAncetre'
CMatriceAleat.prototype.className = 'CMatriceAleat'

CMatriceAleat.prototype.getClone = function (listeSource, listeCible) {
  const ind1 = listeSource.indexOf(this.impProto)
  const minClone = this.min.getClone(listeSource, listeCible)
  const maxClone = this.max.getClone(listeSource, listeCible)
  return new CMatriceAleat(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.nomCalcul, this.n, this.p, minClone, maxClone)
}

CMatriceAleat.prototype.getNatureCalcul = function () {
  return NatCal.NMatriceAleat
}

CMatriceAleat.prototype.positionne = function (infoRandom) {
  this.min.positionne(infoRandom)
  this.max.positionne(infoRandom)
  let existe = this.min.existe && this.max.existe
  let valmin, valmax
  if (existe) {
    valmin = this.min.rendValeur()
    valmax = this.max.rendValeur()
    existe = valmin === Math.round(valmin) && valmax === Math.round(valmax) && (valmin < valmax)
  }
  this.existe = existe
  if (this.existe) {
    // Si on ne demande as un recalcul aléatoire et que la matrice a déjà été calculée on ne la remet pas à jour
    if (!infoRandom && this.mat) return
    let tab = [] // Tableau qui va contenir tous les nombres entiers de min à max
    const tabInit = []
    for (let i = valmin; i <= valmax; i++) {
      tab.push(i)
      tabInit.push(i)
    }
    const mat = []
    for (let i = 0; i < this.n; i++) {
      const lig = []
      mat.push(lig)
      for (let j = 0; j < this.p; j++) {
        // On retire une élément au hasard de tab
        const k = Math.floor(Math.random() * tab.length)
        const val = tab[k]
        tab.splice(k, 1)
        lig.push(val)
        if (tab.length === 0) tab = tabInit.slice(0)
      }
    }
    this.mat = mathjs.matrix(mat) // Matrice contenant les valeurs aléatoires
  }
}

CMatriceAleat.prototype.initialisePourDependance = function () {
  this.min.initialisePourDependance()
  this.max.initialisePourDependance()
}

CMatriceAleat.prototype.depDe = function (p) {
  if (this.elementTestePourDependDe === p) return this.dependDeElementTeste
  return this.memDep(CCalculAncetre.prototype.depDe.call(this, p) || this.min.depDe(p) || this.max.depDe(p))
}

CMatriceAleat.prototype.dependDePourBoucle = function (p) {
  return (p === this) || this.min.dependDePourBoucle(p) || this.max.dependDePourBoucle(p)
}

CMatriceAleat.prototype.read = function (inps, list) {
  CCalculAncetre.prototype.read.call(this, inps, list)
  this.n = inps.readInt()
  this.p = inps.readInt()
  this.min = new CValeur()
  this.min.read(inps, list)
  this.max = new CValeur()
  this.max.read(inps, list)
}

CMatriceAleat.prototype.write = function (oups, list) {
  CCalculAncetre.prototype.write.call(this, oups, list)
  oups.writeInt(this.n)
  oups.writeInt(this.p)
  this.min.write(oups, list)
  this.max.write(oups, list)
}

/**
 * Pour l'interprétation syntaxique, les matrices sont considérées comme des fonctions à deu variables
 * @return {number}
 */
CMatriceAleat.prototype.nombreVariables = function () {
  return 2
}

/**
 * Fonction utilisée dans CalcR. CMatrice, CMatriceAleat sont les seuls objets
 * descendant de CValDyn renvoyant true pour cette fonction (matrices ne résultant pas d'un calcul)
 * @return {boolean}
 */
CMatriceAleat.prototype.estMatriceBase = function () {
  return true
}

/**
 * Fonction utilisée dans CalcR. CMatrice, CMatriceAleat, CMatriceParForm et CCalcMat CCalcMat sont les seuls objets
 * descendant de CValDyn renvoyant true pour cette fonction
 * @return {boolean}
 */
CMatriceAleat.prototype.estMatrice = function () {
  return true
}

/**
 * Fonction renvoyant la représentation LaTeX du résultat de la matrice avec n décimales
 * @param {int} nbdec Le nombre de décimales
 * @return {String}
 */
CMatriceAleat.prototype.latexMat = function (nbdec) {
  return kernel.latexMat(this.mat, nbdec)
}

/**
 * Fonction renvoyant la représentation LaTeX du résultat de la matrice avec appriximation
 * des valeurs par des fractions rationnelles à 10^(-12) près
 * @return {String}
 */
CMatriceAleat.prototype.latexMatFrac = function () {
  return kernel.latexMatFrac(this.mat)
}
