/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatCal = require('../types/NatCal')
const Vect = require('../types/Vect')
const kernel = require('../kernel/kernel')
const CValDyn = require('./CValDyn')
const CCalculAncetre = require('./CCalculAncetre')
module.exports = CMesureAngleOriente

/**
 * Classe représentant la mesure d'un angle orienté donné par trois points.
 * @constructor
 * @extends CMesureAngleGeometrique
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire.
 * @param {boolean} estElementFinal  true si élément final de construction.
 * @param {CPt} a  Premier point (angle aob)
 * @param {CPt} o  Le sommet de l'angle
 * @param {CPt} b  Le troisème point (angle aob)
 * @param {string} nomCalcul  Le nom de la mesure
 * @returns {CMesureAngleOriente}
 */
function CMesureAngleOriente (listeProprietaire, impProto, estElementFinal, a, o, b, nomCalcul) {
  if (arguments.length === 1) CCalculAncetre.call(this, listeProprietaire)
  else {
    CCalculAncetre.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul)
    this.a = a
    this.o = o
    this.b = b
  }
}
CMesureAngleOriente.prototype = new CCalculAncetre()
CMesureAngleOriente.prototype.constructor = CMesureAngleOriente
CMesureAngleOriente.prototype.superClass = 'CCalculAncetre'
CMesureAngleOriente.prototype.className = 'CMesureAngleOriente'

// Ajout version 5.0
CMesureAngleOriente.prototype.numeroVersion = function () {
  return 2
}

CMesureAngleOriente.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.a)
  var ind2 = listeSource.indexOf(this.o)
  var ind3 = listeSource.indexOf(this.b)
  var ind4 = listeSource.indexOf(this.impProto)
  // Ajout version 5.0
  var nomCalc = (this.nomCalcul.length === 0) ? listeCible.genereNomPourCalcul('mesang') : this.nomCalcul
  //
  return new CMesureAngleOriente(listeCible, listeCible.get(ind4, 'CImplementationProto'),
    this.estElementFinal, listeCible.get(ind1, 'CPt'), listeCible.get(ind2, 'CPt'),
    listeCible.get(ind3, 'CPt'), nomCalc)
}
CMesureAngleOriente.prototype.getNatureCalcul = function () {
  return NatCal.NMesureAngleOriente
}
CMesureAngleOriente.prototype.depDe = function (p) {
  if (this.elementTestePourDependDe === p) return this.dependDeElementTeste
  return this.memDep(CCalculAncetre.prototype.depDe.call(this, p) ||
    this.a.depDe(p) || this.o.depDe(p) || this.b.depDe(p))
}
CMesureAngleOriente.prototype.dependDePourBoucle = function (p) {
  return (p === this) || this.a.dependDePourBoucle(p) || this.o.dependDePourBoucle(p) ||
    this.b.dependDePourBoucle(p)
}
CMesureAngleOriente.prototype.rendValeur = function () {
  return this.mesure
}
CMesureAngleOriente.prototype.positionne = function (infoRandom, dimfen) {
  var prosca, deter, tangente, resul

  this.existe = this.a.existe && this.o.existe && this.b.existe &&
    !(this.o.egal(this.a)) && !(this.o.egal(this.b))
  if (!this.existe) return
  // L'orientation sur l'écran est l'inverse de celle du repère
  // choisi.Il faut donc permuter les rôles de U et V
  var v = new Vect(this.o, this.a)
  var u = new Vect(this.o, this.b)
  prosca = u.x * v.x + u.y * v.y
  deter = u.x * v.y - u.y * v.x
  if (prosca === 0) {
    if (deter > 0) resul = Math.PI / 2
    else resul = -Math.PI / 2
  } else {
    tangente = deter / prosca
    if (prosca > 0) resul = Math.atan(tangente)
    else {
      if (deter >= 0) resul = Math.atan(tangente) + Math.PI
      else resul = Math.atan(tangente) - Math.PI
    }
  }
  // Nouveau pour version 1.3.6 : on regarde si l'angle est très proche de - pi. Si oui on renvoie pi
  if (Math.abs(resul + Math.PI) < 1e-13)resul = Math.PI
  if (this.listeProprietaire.uniteAngle === kernel.uniteAngle.radian) this.mesure = resul
  else this.mesure = resul * kernel.ConvRadDeg
}
CMesureAngleOriente.prototype.confonduAvec = function (p) {
  if (p.className === this.className) {
    return ((this.o === p.o) && (this.a === p.a) && (this.b === p.b))
  } else return false
}
CMesureAngleOriente.prototype.remplacePoint = function (ancienPoint, nouveauPoint) {
  if (this.a === ancienPoint) this.a = nouveauPoint
  if (this.o === ancienPoint) this.o = nouveauPoint
  if (this.b === ancienPoint) this.b = nouveauPoint
}

// Ajouts version 5.0 car maintenant dérive de CCalculAncetre
CMesureAngleOriente.prototype.read = function (inps, list) {
  if (this.nVersion >= 2) CCalculAncetre.prototype.read.call(this, inps, list)
  else {
    CValDyn.prototype.read.call(this, inps, list)
    if (this.listeProprietaire.className === 'CPrototype') this.nomCalcul = 'ang'
    else this.nomCalcul = this.listeProprietaire.genereNomPourCalcul('ang')
  }
  var ind1 = inps.readInt()
  var ind2 = inps.readInt()
  var ind3 = inps.readInt()
  this.a = list.get(ind1, 'CPt')
  this.o = list.get(ind2, 'CPt')
  this.b = list.get(ind3, 'CPt')
}
CMesureAngleOriente.prototype.write = function (oups, list) {
  CCalculAncetre.prototype.write.call(this, oups, list)
  var ind1 = list.indexOf(this.a)
  oups.writeInt(ind1)
  var ind2 = list.indexOf(this.o)
  oups.writeInt(ind2)
  var ind3 = list.indexOf(this.b)
  oups.writeInt(ind3)
}
