/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Complexe = require('../types/Complexe')
const kernel = require('../kernel/kernel')
const CCbGlob = require('../kernel/CCbGlob')
const CSommeDansFormule = require('./CSommeDansFormule')
module.exports = CProduitDansFormule

/**
 * Classe représentant un produit indicé dans un arbre binaire de calcul.
 * @constructor
 * @extends CCb
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CCb} calculASommer  Le calcul à multiplier
 * @param {CCb} bornea  L'indice de départ.
 * @param {CCb} borneb  L'indice de fin.
 * @param {indiceFin} pas  Le pas à ajouter à l'indice pour chaque somme (1 la plupart du temps)
 * @param {String} nomVariable  Le nom choisi pour la variable de sommation.
 * @returns {CProduitDansFormule}
 */
function CProduitDansFormule (listeProprietaire, calculASommer, bornea, borneb, pas, nomVariable) {
  if (arguments.length === 1) CSommeDansFormule.call(this, listeProprietaire)
  else CSommeDansFormule.call(this, listeProprietaire, calculASommer, bornea, borneb, pas, nomVariable)
}
CProduitDansFormule.prototype = new CSommeDansFormule()
CProduitDansFormule.prototype.constructor = CProduitDansFormule
CProduitDansFormule.prototype.superClass = 'CSommeDansFormule'
CProduitDansFormule.prototype.className = 'CProduitDansFormule'

CProduitDansFormule.prototype.nature = function () {
  return CCbGlob.natProduitDansFormule
}

CProduitDansFormule.prototype.getClone = function (listeSource, listeCible) {
  return new CProduitDansFormule(listeCible, this.calculASommer.getClone(listeSource, listeCible),
    this.indiceDebut.getClone(listeSource, listeCible), this.indiceFin.getClone(listeSource, listeCible),
    this.pas.getClone(listeSource, listeCible), this.nomVariable)
}

CProduitDansFormule.prototype.getCopie = function () {
  return new CProduitDansFormule(this.listeProprietaire, this.calculASommer.getCopie(), this.indiceDebut.getCopie(),
    this.indiceFin.getCopie(), this.pas.getCopie(), this.nomVariable)
}
/**
 * Renvoie le résultat réel du produit.
 * @param {boolean} infoRandom  true si les calculs aléatoires doivent être relancés
 * @returns {number}
 */
CProduitDansFormule.prototype.resultat = function (infoRandom) {
  var val = new Array(1)
  var MIN_VALUE = kernel.MIN_VALUE
  var MAX_VALUE = kernel.MAX_VALUE
  var dinddeb = this.indiceDebut.resultat(infoRandom)
  if ((dinddeb > MAX_VALUE) || (dinddeb < MIN_VALUE) || (dinddeb !== Math.floor(dinddeb))) throw new Error(kernel.erreurCalculException)
  var dindfin = this.indiceFin.resultat(infoRandom)
  if ((dindfin > MAX_VALUE) || (dindfin < MIN_VALUE) || (dindfin !== Math.floor(dindfin)) ||
      (dindfin < dinddeb)) throw new Error(kernel.erreurCalculException)
  var dpas = this.pas.resultat(infoRandom)
  if ((dpas > MAX_VALUE) || (dpas < MIN_VALUE) ||
      (dpas !== Math.floor(dpas)) || (dpas <= 0)) throw new Error(kernel.erreurCalculException)
  this.listeProprietaire.nombreIterations += (dindfin - dinddeb) / dpas
  if (this.listeProprietaire.nombreIterations > CCbGlob.nombreMaxiIterations) throw new Error(kernel.erreurCalculException)
  var somme = 1
  for (var i = dinddeb; i <= dindfin; i += dpas) {
    val[0] = i
    somme *= this.calculASommer.resultatFonction(infoRandom, val)
  }
  return somme
}
/**
 * Renvoie le résultat complexe du produit dans le complee zres.
 * @param {boolean} infoRandom  true si les calculs aléatoires doivent être relancés
 * @param {Complexe} zRes
 * @returns {undefined}
 */
CProduitDansFormule.prototype.resultatComplexe = function (infoRandom, zRes) {
  var MIN_VALUE = kernel.MIN_VALUE
  var MAX_VALUE = kernel.MAX_VALUE
  var val = new Array(1)
  val[0] = new Complexe()
  var inddeb = new Complexe()
  this.indiceDebut.resultatComplexe(infoRandom, inddeb)
  if (inddeb.y !== 0) throw new Error(kernel.erreurCalculException)
  var dinddeb = inddeb.x
  if ((dinddeb > MAX_VALUE) || (dinddeb < MIN_VALUE) || (dinddeb !== Math.floor(dinddeb))) throw new Error(kernel.erreurCalculException)
  var indfin = new Complexe()
  this.indiceFin.resultatComplexe(infoRandom, indfin)
  if (inddeb.y !== 0) throw Error(kernel.erreurCalculException)
  var dindfin = indfin.x
  if ((dindfin > MAX_VALUE) || (dindfin < MIN_VALUE) || (dindfin !== Math.floor(dindfin)) ||
      (dindfin < dinddeb)) throw new Error(kernel.erreurCalculException)
  var valpas = new Complexe()
  this.pas.resultatComplexe(infoRandom, valpas)
  if (valpas.y !== 0) throw new Error(kernel.erreurCalculException)
  var dvalpas = valpas.x
  if ((dvalpas > MAX_VALUE) || (dvalpas < MIN_VALUE) ||
      (dvalpas !== Math.floor(dvalpas)) || (dvalpas <= 0)) throw new Error(kernel.erreurCalculException)
  this.listeProprietaire.nombreIterations += (dindfin - dinddeb) / dvalpas
  if (this.listeProprietaire.nombreIterations > CCbGlob.nombreMaxiIterations) throw new Error(kernel.erreurCalculException)
  zRes.x = 1
  zRes.y = 0
  var z = new Complexe()
  for (var i = inddeb.x; i <= indfin.x; i += valpas.x) {
    val[0].x = i
    val[0].y = 0
    this.calculASommer.resultatFonctionComplexe(infoRandom, val, z)
    zRes.produitComp(z, zRes)
  }
}

CProduitDansFormule.prototype.resultatFonctionComplexe = function (infoRandom, valeurParametre, zRes) {
  var i, val, dinddeb, dindfin, dvalpas, inddeb, indfin, z
  var MIN_VALUE = kernel.MIN_VALUE
  var MAX_VALUE = kernel.MAX_VALUE
  if (valeurParametre instanceof Array) {
    var n = valeurParametre.length
    val = new Array(n + 1)
    for (i = 0; i < n; i++) val[i] = new Complexe(valeurParametre[i])
    val[n] = new Complexe()
  } else {
    val = new Array(2)
    val[0] = valeurParametre
    val[1] = new Complexe()
  }
  inddeb = new Complexe()
  this.indiceDebut.resultatFonctionComplexe(infoRandom, valeurParametre, inddeb)
  if (inddeb.y !== 0) throw new Error(kernel.erreurCalculException)
  dinddeb = inddeb.x
  if ((dinddeb > MAX_VALUE) || (dinddeb < MIN_VALUE) || (dinddeb !== Math.floor(dinddeb))) throw new Error(kernel.erreurCalculException)
  indfin = new Complexe()
  this.indiceFin.resultatFonctionComplexe(infoRandom, valeurParametre, indfin)
  if (indfin.y !== 0) throw new Error(kernel.erreurCalculException)
  dindfin = indfin.x
  if ((dindfin > MAX_VALUE) || (dindfin < MIN_VALUE) || (dindfin !== Math.floor(dindfin)) ||
      (dindfin < dinddeb)) throw new Error(kernel.erreurCalculException)
  var valpas = new Complexe()
  this.pas.resultatFonctionComplexe(infoRandom, valeurParametre, valpas)
  if (valpas.y !== 0) throw new Error(kernel.erreurCalculException)
  dvalpas = valpas.x
  if ((dvalpas > MAX_VALUE) || (dvalpas < MIN_VALUE) ||
      (dvalpas !== Math.floor(dvalpas)) || (dvalpas <= 0)) throw new Error(kernel.erreurCalculException)
  this.listeProprietaire.nombreIterations += (dindfin - dinddeb) / dvalpas
  if (this.listeProprietaire.nombreIterations > CCbGlob.nombreMaxiIterations) throw new Error(kernel.erreurCalculException)
  zRes.x = 1
  zRes.y = 0
  z = new Complexe()
  for (i = inddeb.x; i <= indfin.y; i += valpas.x) {
    val[n].x = i
    val[n].y = 0
    this.calculASommer.resultatFonctionComplexe(infoRandom, val, z)
    zRes.produitComp(z, zRes)
  }
}

CProduitDansFormule.prototype.chaineCalcul = function (varFor) {
  var varFor2, i
  var n = 0
  if (varFor === null) varFor2 = new Array(1)
  else {
    n = varFor.length
    varFor2 = new Array(n + 1)
  }
  for (i = 0; i < n; i++) varFor2[i] = varFor[i]
  varFor2[n] = this.nomVariable
  return kernel.getStr('produit') + '(' + this.calculASommer.chaineCalculSansPar(varFor2) + ',' +
    this.nomVariable + ',' + this.indiceDebut.chaineCalculSansPar(varFor) + ',' +
    this.indiceFin.chaineCalculSansPar(varFor) + ',' + this.pas.chaineCalculSansPar(varFor) + ')'
}

CProduitDansFormule.prototype.chaineLatex = function (varFor) {
  var varFor2, i, up
  var n = 0
  if (varFor === null) varFor2 = new Array(1)
  else {
    n = varFor.length
    varFor2 = new Array(n + 1)
  }
  for (i = 0; i < n; i++) varFor2[i] = varFor[i]
  varFor2[n] = this.nomVariable
  up = ''
  if (this.pas.nature() === CCbGlob.natConstante) {
    if (this.pas.valeur === 1) up = this.indiceFin.chaineLatex(varFor)
  }
  if (up.length === 0) up = this.indiceFin.chaineLatex(varFor) + '\\text{ ' + kernel.getStr('step') + ' }' + this.pas.chaineLatex(varFor)
  return '\\prod\\limits_{' + this.nomVariable + '=' + this.indiceDebut.chaineLatexSansPar(varFor) + '}' + '^{' + up + '}' + '{' + this.calculASommer.chaineLatex(varFor2) + '}'
}

CProduitDansFormule.prototype.deriveePossible = function (indiceVariable) {
  return !this.calculASommer.dependDeVariable(indiceVariable) &&
    !this.indiceDebut.dependDeVariable(indiceVariable) && !this.indiceFin.dependDeVariable(indiceVariable) &&
    !this.pas.dependDeVariable(indiceVariable)
}
