/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CSurface = require('./CSurface')
const CSegment = require('./CSegment')
const CPointBase = require('./CPointBase')
module.exports = CSurfaceDeuxLieux

/**
 * Classe représentant une surface délimitée par deux mieux de points.
 * @constructor
 * @extends CSurface
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire.
 * @param {boolean} estElementFinal  true si élément final de construction.
 * @param {Color} couleur  La couleur.
 * @param {boolean} masque  true si l'objet est masqué.
 * @param {StyleRemplissage} styleRemplissage  Entier donnant le style de remplissage.
 * @param {CElementGraphique} bord  Le premier bord de la surface (lieu de points).
 * @param {CLieuDeBase} deuxiemeLieu  Le deuxième bord (lieu de points).
 * @returns {CSurfaceDeuxLieux}
 */
function CSurfaceDeuxLieux (listeProprietaire, impProto, estElementFinal, couleur, masque, styleRemplissage,
  bord, deuxiemeLieu) {
  if (arguments.length === 1) CSurface.call(this, listeProprietaire)
  else {
    CSurface.call(this, listeProprietaire, impProto, estElementFinal, couleur,
      masque, styleRemplissage, bord)
    this.deuxiemeLieu = deuxiemeLieu
    this.abscisses = new Array(bord.infoLieu.nombreDePoints + 4)
    this.ordonnees = new Array(bord.infoLieu.nombreDePoints + 4)
  }
  this.pointDebutLieu1 = new CPointBase()
  this.pointDebutLieu2 = new CPointBase()
  this.pointFinLieu1 = new CPointBase()
  this.pointFinLieu2 = new CPointBase()
  this.segmentDebut = new CSegment(listeProprietaire, this.pointDebutLieu1, this.pointDebutLieu2)
  this.segmentFin = new CSegment(listeProprietaire, this.pointFinLieu1, this.pointFinLieu2)
}
CSurfaceDeuxLieux.prototype = new CSurface()
CSurfaceDeuxLieux.prototype.constructor = CSurfaceDeuxLieux
CSurfaceDeuxLieux.prototype.superClass = 'CSurface'
CSurfaceDeuxLieux.prototype.className = 'CSurfaceDeuxLieux'

CSurfaceDeuxLieux.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.bord)
  var ind2 = listeSource.indexOf(this.deuxiemeLieu)
  var ind3 = listeSource.indexOf(this.impProto)
  return new CSurfaceDeuxLieux(listeCible, listeCible.get(ind3, 'CImplementationProto'),
    this.estElementFinal, this.couleur, this.masque, this.styleRemplissage,
    listeCible.get(ind1, 'CLieuDeBase'), listeCible.get(ind2, 'CLieuDeBase'))
}
CSurfaceDeuxLieux.prototype.setClone = function (ptel) {
  CSurface.prototype.setClone.call(this, ptel)
  if (ptel.existe) {
    this.nombrePointsTraces = ptel.nombrePointsTraces
    for (var i = 0; i < this.nombrePointsTraces; i++) {
      this.abscisses[i] = ptel.abscisses[i]
      this.ordonnees[i] = ptel.ordonnees[i]
    }
  }
}
CSurfaceDeuxLieux.prototype.ajouteAntecedents = function (liste) {
  liste.add(this.bord)
  liste.add(this.deuxiemeLieu)
}
CSurfaceDeuxLieux.prototype.confonduAvec = function (p) {
  var res = CSurface.prototype.confonduAvec.call(this, p)
  if (!res) return false
  return this.deuxiemeLieu === p.deuxiemeLieu
}
CSurfaceDeuxLieux.prototype.depDe = function (p) {
  if (this.elementTestePourDependDe === p) return this.dependDeElementTeste
  return this.memDep(CSurface.prototype.depDe.call(this, p) || this.deuxiemeLieu.depDe(p))
}
CSurfaceDeuxLieux.prototype.dependDePourBoucle = function (p) {
  return CSurface.prototype.dependDePourBoucle.call(this, p) || this.deuxiemeLieu.dependDePourBoucle(p)
}
CSurfaceDeuxLieux.prototype.dependDePourCapture = function (p) {
  return CSurface.prototype.dependDePourCapture.call(this, p) || this.deuxiemeLieu.dependDePourCapture(p)
}
CSurfaceDeuxLieux.prototype.positionne = function (infoRandom, dimfen) {
  var i, x1, x2, x3, x4, y1, y2, y3, y4, ideb, deb, indicePremierPointLieu1, indiceDernierPointLieu1, indicePremierPointLieu2,
    indiceDernierPointLieu2, nombrePointsTracesPremierLieu, nombrePointsTracesDeuxiemeLieu
  this.existe = (this.bord.existe) && (this.deuxiemeLieu.existe)
  if (!this.existe) return
  // Les surfaces délimitant le bord ne doivent être formées
  // que d'une seule ligne
  if (!(this.bord.contientUneSeuleLigne() && this.deuxiemeLieu.contientUneSeuleLigne())) {
    this.existe = false
    return
  }

  nombrePointsTracesPremierLieu = this.bord.infoLignes[this.bord.indicePremiereLigne].nombrePoints
  nombrePointsTracesDeuxiemeLieu = this.deuxiemeLieu.infoLignes[this.deuxiemeLieu.indicePremiereLigne].nombrePoints
  this.nombrePointsTraces = nombrePointsTracesPremierLieu + nombrePointsTracesDeuxiemeLieu
  indicePremierPointLieu1 = this.bord.infoLignes[this.bord.indicePremiereLigne].indicePremierPoint
  indiceDernierPointLieu1 = this.bord.infoLignes[this.bord.indicePremiereLigne].indicePremierPoint +
      this.bord.infoLignes[this.bord.indicePremiereLigne].nombrePoints - 1
  indicePremierPointLieu2 = this.deuxiemeLieu.infoLignes[this.deuxiemeLieu.indicePremiereLigne].indicePremierPoint
  indiceDernierPointLieu2 = this.deuxiemeLieu.infoLignes[this.deuxiemeLieu.indicePremiereLigne].indicePremierPoint +
      this.deuxiemeLieu.infoLignes[this.deuxiemeLieu.indicePremiereLigne].nombrePoints - 1
  x1 = this.bord.absPoints[indicePremierPointLieu1]
  y1 = this.bord.ordPoints[indicePremierPointLieu1]
  x4 = this.bord.absPoints[indiceDernierPointLieu1]
  y4 = this.bord.ordPoints[indiceDernierPointLieu1]
  x3 = this.deuxiemeLieu.absPoints[indiceDernierPointLieu2]
  y3 = this.deuxiemeLieu.ordPoints[indiceDernierPointLieu2]
  x2 = this.deuxiemeLieu.absPoints[indicePremierPointLieu2]
  y2 = this.deuxiemeLieu.ordPoints[indicePremierPointLieu2]
  this.pointDebutLieu1.placeEn(x1, y1)
  this.pointFinLieu2.placeEn(x4, y4)
  // On recopie les coordonnées du premier lieu
  ideb = this.bord.infoLignes[this.bord.indicePremiereLigne].indicePremierPoint
  for (i = 0; i < nombrePointsTracesPremierLieu; i++) {
    this.abscisses[i] = this.bord.absPoints[ideb + i]
    this.ordonnees[i] = this.bord.ordPoints[ideb + i]
  }
  deb = this.deuxiemeLieu.infoLignes[this.deuxiemeLieu.indicePremiereLigne].indicePremierPoint
  for (i = 0; i <= nombrePointsTracesDeuxiemeLieu - 1; i++) {
    this.abscisses[nombrePointsTracesPremierLieu + i] = this.deuxiemeLieu.absPoints[deb + nombrePointsTracesDeuxiemeLieu - i - 1]
    this.ordonnees[nombrePointsTracesPremierLieu + i] = this.deuxiemeLieu.ordPoints[deb + nombrePointsTracesDeuxiemeLieu - i - 1]
  }
  this.pointDebutLieu2.placeEn(x2, y2)
  this.pointFinLieu1.placeEn(x3, y3)
  this.segmentDebut.positionne(infoRandom, dimfen)
  this.segmentFin.positionne(infoRandom, dimfen)
  this.existe = (this.segmentDebut.existe) && (this.segmentFin.existe)
}
/**
 * Fonction renvoyant une chaîne de caractères utilisée dans le svg élément
 * représentant la surface.
 * @returns {String}
 */
CSurfaceDeuxLieux.prototype.creePoints = function () {
  var j; var points = ''
  for (j = 0; j < this.nombrePointsTraces; j++) {
    points = points + this.abscisses[j] + ','
    points = points + this.ordonnees[j] + ' '
  }
  return points
}
CSurfaceDeuxLieux.prototype.read = function (inps, list) {
  CSurface.prototype.read.call(this, inps, list)
  var ind1 = inps.readInt()
  this.deuxiemeLieu = list.get(ind1, 'CLieuDeBase')
  if (list.className !== 'CPrototype') {
    this.abscisses = new Array(this.bord.infoLieu.nombreDePoints + this.deuxiemeLieu.infoLieu.nombreDePoints + 2)
    this.ordonnees = new Array(this.bord.infoLieu.nombreDePoints + this.deuxiemeLieu.infoLieu.nombreDePoints + 2)
  }
}
CSurfaceDeuxLieux.prototype.write = function (oups, list) {
  CSurface.prototype.write.call(this, oups, list)
  var ind1 = list.indexOf(this.deuxiemeLieu)
  oups.writeInt(ind1)
}
