/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatCal = require('../types/NatCal')
const CCalculAncetre = require('./CCalculAncetre')
const CValeur = require('./CValeur')
module.exports = CTestEquivalence

/**
 * Calcul testant si deux formules de calcul ou de fonctions sont équivalentes
 * au sens en tenant compte de commutativité de la multiplication et de l'addition.
 * Deux calculs comme a/b ou 1/b*a sont aussi considérés comme équivalents.
 * @constructor
 * @extends CCalculAncetre
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou la construction propriétaire.
 * @param {boolean} estElementFinal  true si élément final de construction.
 * @param {String} nomCalcul  Le nom du calcul.
 * @param {CCalcul} calcul1  Le premier calcul (ou fonction) testé.
 * @param {CCalcul} calcul2  Le deuxième calcul (ou fonction) testé.
 * @param {boolean} remplacementValeurs1  Si true, les résultats de valeurs utilisés dans le
 * calcul calcul1 sont remplacés avant de tester l'équivalence des formules.
 * @param {boolean} remplacementValeurs2  Si true, les résultats de valeurs utilisés dans le
 * calcul calcul2 sont remplacés avant de tester l'équivalence des formules.
 * @param {CValeur} test  Si différent de 1, le calcul est
 * @param {boolean} equivalenceDecimalFracIrr  true si on accepte aussi bien les nombres décimaux
 * que leur fraction irréductible équivalente (au plus 9 décimales)
 * considéré comme non existant. Permet d'éviter la comparaison quand une condition n'est pas vérifiée.
 * @returns {CTestEquivalence}
 */
function CTestEquivalence (listeProprietaire, impProto, estElementFinal, nomCalcul,
  calcul1, calcul2, remplacementValeurs1, remplacementValeurs2, test, equivalenceDecimalFracIrr) {
  if (arguments.length === 0) return // Ajout version WebPack
  if (arguments.length === 1) CCalculAncetre.call(this, listeProprietaire)
  else {
    if (arguments.length === 3) CCalculAncetre.call(this, listeProprietaire, impProto, estElementFinal)
    else {
      CCalculAncetre.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul)
      this.calcul1 = calcul1
      this.calcul2 = calcul2
      this.remplacementValeurs1 = remplacementValeurs1
      this.remplacementValeurs2 = remplacementValeurs2
      this.test = test
      this.equivalenceDecimalFracIrr = equivalenceDecimalFracIrr
    }
  }
  this.existe = true
  this.dejaPositionne = false
}
CTestEquivalence.prototype = new CCalculAncetre()
CTestEquivalence.prototype.constructor = CTestEquivalence
CTestEquivalence.prototype.superClass = 'CCalculAncetre'
CTestEquivalence.prototype.className = 'CTestEquivalence'

CTestEquivalence.prototype.numeroVersion = function () {
  return 3
}

CTestEquivalence.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.impProto)
  var ind2 = listeSource.indexOf(this.calcul1)
  var ind3 = listeSource.indexOf(this.calcul2)
  var testClone = this.test.getClone(listeSource, listeCible)
  return new CTestEquivalence(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.nomCalcul, listeCible.get(ind2, 'CCalcul'),
    listeCible.get(ind3, 'CCalcul'), this.remplacementValeurs1, this.remplacementValeurs2, testClone,
    this.equivalenceDecimalFracIrr)
}
CTestEquivalence.prototype.getNatureCalcul = function () {
  return NatCal.NTestEquivalence
}
CTestEquivalence.prototype.rendValeur = function () {
  return this.resultat
}
CTestEquivalence.prototype.positionne = function (infoRandom, dimfen) {
  // Existe toujours égal à true pour cet objet
  this.test.positionne(infoRandom, dimfen)
  if (!this.test.existe) {
    this.resultat = 0
    return
  }
  if (this.test.rendValeur() !== 1) {
    this.resultat = 0
    return
  }
  if (!this.dejaPositionne) {
    if (this.remplacementValeurs1) {
      this.calculeq1 =
      this.calcul1.calcul.calculAvecValeursRemplacees(false).calculNormalise(true, true, this.equivalenceDecimalFracIrr)
    } else this.calculeq1 = this.calcul1.calcul.calculNormalise(this.remplacementValeurs1, true, this.equivalenceDecimalFracIrr)
    if (this.remplacementValeurs2) {
      this.calculeq2 =
      this.calcul2.calcul.calculAvecValeursRemplacees(false).calculNormalise(true, true, this.equivalenceDecimalFracIrr)
    } else this.calculeq2 = this.calcul2.calcul.calculNormalise(this.remplacementValeurs2, true, this.equivalenceDecimalFracIrr)
    this.dejaPositionne = true // Ajout version 4.7.2
  }
  if (!this.calcul1.existe || !this.calcul2.existe) {
    this.resultat = 0
    return
  }
  if (this.calculeq1.estEquivalentA(this.calculeq2)) this.resultat = 1; else this.resultat = 0
}
CTestEquivalence.prototype.positionneFull = function (infoRandom, dimfen) {
  this.test.positionne(infoRandom, dimfen)
  if (!this.test.existe) {
    this.resultat = 0
    return
  }
  if (this.test.rendValeur() !== 1) {
    this.resultat = 0
    return
  }
  if (this.remplacementValeurs1) {
    this.calculeq1 =
    this.calcul1.calcul.calculAvecValeursRemplacees(false).calculNormalise(true, true, this.equivalenceDecimalFracIrr)
  } else this.calculeq1 = this.calcul1.calcul.calculNormalise(this.remplacementValeurs1, true, this.equivalenceDecimalFracIrr)
  if (this.remplacementValeurs2) {
    this.calculeq2 =
    this.calcul2.calcul.calculAvecValeursRemplacees(false).calculNormalise(true, true, this.equivalenceDecimalFracIrr)
  } else this.calculeq2 = this.calcul2.calcul.calculNormalise(this.remplacementValeurs2, true, this.equivalenceDecimalFracIrr)
  this.dejaPositionne = true
  if (!this.calcul1.existe || !this.calcul2.existe) {
    this.resultat = 0
    return
  }
  if (this.calculeq1.estEquivalentA(this.calculeq2)) this.resultat = 1; else this.resultat = 0
}
CTestEquivalence.prototype.depDe = function (p) {
  if (this.elementTestePourDependDe === p) return this.dependDeElementTeste
  return this.memDep(CCalculAncetre.prototype.depDe.call(this, p) ||
    this.calcul1.depDe(p) || this.calcul2.depDe(p) || this.test.depDe(p))
}
CTestEquivalence.prototype.dependDePourBoucle = function (p) {
  return (p === this) || this.calcul1.dependDePourBoucle(p) || this.calcul2.dependDePourBoucle(p) ||
  this.test.dependDePourBoucle(p)
}
CTestEquivalence.prototype.metAJour = function () {
  this.dejaPositionne = false
}
CTestEquivalence.prototype.read = function (inps, list) {
  CCalculAncetre.prototype.read.call(this, inps, list)
  var ind1 = inps.readInt()
  this.calcul1 = list.get(ind1, 'CCalcul')
  var ind2 = inps.readInt()
  this.calcul2 = list.get(ind2, 'CCalcul')
  this.remplacementValeurs1 = inps.readBoolean()
  this.remplacementValeurs2 = inps.readBoolean()
  this.test = new CValeur(this.listeProprietaire)
  if (this.nVersion < 2) this.test.donneValeur(1)
  else this.test.read(inps, list)
  if (this.nVersion < 3) this.equivalenceDecimalFracIrr = false
  else this.equivalenceDecimalFracIrr = inps.readBoolean()
}
CTestEquivalence.prototype.write = function (oups, list) {
  CCalculAncetre.prototype.write.call(this, oups, list)
  var ind1 = list.indexOf(this.calcul1)
  oups.writeInt(ind1)
  var ind2 = list.indexOf(this.calcul2)
  oups.writeInt(ind2)
  oups.writeBoolean(this.remplacementValeurs1)
  oups.writeBoolean(this.remplacementValeurs2)
  this.test.write(oups, list)
  oups.writeBoolean(this.equivalenceDecimalFracIrr)
}
