/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const Nat = require('../types/Nat')
const NatObj = require('../types/NatObj')
const OutilObjetPar2Objets = require('./OutilObjetPar2Objets')
const CSurfaceDeuxLieux = require('../objets/CSurfaceDeuxLieux')
const CImplementationProto = require('../objets/CImplementationProto')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = OutilCouronne

/**
 * Outil servant à créer une surface délimitée par un polygône, un cercle, un arc de cercle ou un lieu fermé
 * @param app L'application propriétaire
 * @constructor
 */
function OutilCouronne (app) {
  var nat = Nat.or(NatObj.NLieu, NatObj.NCercle, NatObj.NPolygone)
  OutilObjetPar2Objets.call(this, app, 'Couronne', 32873, true, nat, nat)
}

OutilCouronne.prototype = new OutilObjetPar2Objets()

OutilCouronne.prototype.select = function () {
  // On ne doit pas pouvoir désigner des lieux de points non fermés
  OutilObjetPar2Objets.prototype.select.call(this)
  var list = this.app.listePr
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    if (el.estDeNature(NatObj.NLieu) && !el.infoLieu.fermeOuNon) this.excluDeDesignation(el)
  }
}

OutilCouronne.prototype.indication = function (ind) {
  switch (ind) {
    case 1 : return 'indCour'
    case 2 : return 'indCour'
  }
}

OutilCouronne.prototype.creeObjet = function () {
  var proto; var surf
  var app = this.app
  var list = app.listePr
  var docConsAv = app.docConsAv
  var ob1 = this.ob1; var ob2 = this.ob2
  // Cas où l'utilisateur a cliqué sur deux lieux fermés : Inutile d'utiliser une macro-construction
  if (ob1.estDeNature(NatObj.NLieu) && ob2.estDeNature(NatObj.NLieu)) {
    surf = new CSurfaceDeuxLieux(app.listePr, null, false, app.getCouleur(), false, app.getStyleRemplissage(), ob1, ob2)
    app.ajouteElement(surf)
    var verif = app.verifieDernierElement(1)
    if (verif) {
      // L'élément créé existe bien et a été ajouté à la liste
      surf.creeAffichage(this.app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
      this.annuleClignotement()
      this.saveFig()
    } else new AvertDlg(app, 'DejaCree')

    this.reselect()
    return
  } else if (this.sontDeNat(NatObj.NCercle, NatObj.NCercle)) proto = docConsAv.getPrototype('Couronne')
  else if (this.sontDeNat(NatObj.NCercle, NatObj.NPolygone)) proto = docConsAv.getPrototype('CouronneCerclePoly')
  else if (this.sontDeNat(NatObj.NPolygone, NatObj.NPolygone)) proto = docConsAv.getPrototype('CouronnePolys')
  else if (this.sontDeNat(NatObj.NCercle, NatObj.NLieu)) proto = docConsAv.getPrototype('CouronneCercleLieu')
  else if (this.sontDeNat(NatObj.NPolygone, NatObj.NLieu)) proto = docConsAv.getPrototype('CouronnePolyLieu')

  // On affecte les éléments sources au prototype
  proto.get(0).elementAssocie = this.ob1
  proto.get(1).elementAssocie = this.ob2
  var impProto = new CImplementationProto(list, proto)
  impProto.implemente(app.dimf, proto)
  impProto.nomProto = kernel.getStr('Couronne')

  surf = impProto.dernierFinal(NatObj.NSurface)
  surf.donneCouleur(app.getCouleur())
  surf.donneStyleRemplissage(app.svgFigure, app.getStyleRemplissage())
  var indImpProto = list.indexOf(impProto)
  list.positionne(false, app.dimf)
  list.setReady4MathJax()
  list.afficheTout(indImpProto, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
  // Corrigé version 5.6.4
  if (app.estExercice) app.listePourConst = app.listePourConstruction()

  app.gestionnaire.enregistreFigureEnCours('Couronne')
}

/**
 * Fonction renvoyant true si this.ob1 et this.obj sont de nature nat1 et nat2.
 * Dans le cas où c'est le cas, this.ob1 et this.ob2 sont remaniés si nécessaire de façon à ce que
 * Si au moins un des deux est un cercle, this.ob1 pointe sur le cercle
 * Si aucun n'est un cercle et si au moins un des deux est un polygone, this.ob1 pointe sur le polygone
 * @param nat1
 * @param nat2
 */
OutilCouronne.prototype.sontDeNat = function (nat1, nat2) {
  var ob1 = this.ob1; var ob2 = this.ob2
  if ((ob1.estDeNature(nat1) && ob2.estDeNature(nat2)) || (ob1.estDeNature(nat2) && ob2.estDeNature(nat1))) {
    if (ob1.estDeNature(NatObj.NCercle)) return true
    if (ob2.estDeNature(NatObj.NCercle)) {
      this.ob1 = ob2
      this.ob2 = ob1
    } else {
      if (ob1.estDeNature(NatObj.NPolygone)) return true
      if (ob2.estDeNature(NatObj.NPolygone)) {
        this.ob1 = ob2
        this.ob2 = ob1
      }
    }
    return true
  } else return false
}
