const path = require('path')

const autoprefixer = require('autoprefixer')
const TerserPlugin = require('terser-webpack-plugin')
const webpack = require('webpack')
const { CleanWebpackPlugin } = require('clean-webpack-plugin')

const MiniCssExtractPlugin = require('mini-css-extract-plugin')

// On récupère la même conf babel que celle de package.json
const babelConfig = require('./package').babel
const babelLoader = {
  loader: 'babel-loader',
  options: babelConfig
}

/**
 * Pour lancer localement avec webpack-dev-server, c'est `npm start`
 * (puis http://localhost:8081, ou http://localhost:8081/#playerXXX ou http://localhost:8081/#XXX
 * où XXX est une figure en base64)
 *
 * Pour la compilation des js à copier sur mathgraph32.org c'est `npm run build`
 */

const host = 'localhost'
const port = 8081

// pour savoir si on est lancé par webpack-dev-server
const isDevServer = process.argv[1].includes('dev-server')
// passer --debug pour ne pas avoir de minification (ou lancer avec npm start)
// et pouvoir fonctionner en local (sinon les dépendances seront chargées sur mathgraph32.org)
const isDev = process.argv.includes('--debug') || isDevServer
// Un paramètre --local pour générer une version locale minimisée dans un seul fichier
let isLocal = !isDevServer && process.argv.includes('--local')
// pour une génération locale, on autorise aussi un path en paramètre
let localPath
// avec `webpack --localPath foo` ça va faire un build dans www/build/mtgApp/local/foo
// et ce dossier foo pourra être copié à la racine de n'importe quel site
// pour pouvoir appeler http://autreSite/foo/mtgLoad.min.js
if (!isDevServer && process.argv.includes('--localPath')) {
  const pos = process.argv.indexOf('--localPath')
  localPath = process.argv[pos + 1]
  isLocal = true
}
let isAllInOne = process.argv.includes('--all-in-one')
// pour le path absolu
const isPreprod = process.argv.includes('--preprod')

// Un paramètre pour générer la version spéciale electron
const isElectron = process.argv.indexOf('--electron') !== -1
if (isElectron) {
  // qui impose local & allInOne, sauf avec devServer (où on laisse la conf par défaut)
  if (!isDevServer) {
    isLocal = true
    isAllInOne = true
  }
} else if (isLocal) {
  isAllInOne = true
}

// puis tout ce qui en dépend, le dossier de sortie
let buildPath
// préfixe à utiliser pour nos chargements (quand un module en charge un autre)
// si un de nos js est chargé en cross domain il faut impérativement que cette valeur
// soit une url absolue avec le domaine où ce js sera chargé (pour qu'il puisse charger
// ses dépendances depuis le domaine d'appel)
let publicPath

// des booléens qui dépendent du reste, pour la lisibilité
const hasMinify = !isDev
// pour le sourceMap, toujours sauf electron en prod
const hasSourceMap = !isElectron || isDev
// pour le purge du dossier de build
// jamais en prod|preprod (en prod on veut garder le build précédent,
//   c'est le postDeploy qui nettoie les plus vieux en gardant le dernier
//   et en préprod on passe juste après un build de dev qu'il ne faut pas supprimer)
const hasPurge = !isDevServer && (isDev || isLocal)

// set buildPath
if (isElectron) {
  // pas dans www/build mais directement dans le dossier electron
  buildPath = 'electron/build'
} else if (isLocal) {
  buildPath = 'www/build/local'
  // si on veut copier tous les js dans un dossier mtg qui sera accessible
  // via https://domaine.tld/path/to/mtg/*.js
  // il faut passer un --localPath path/to/mtg
  if (localPath) {
    // passer du --localPath build/local/… pour tester nos js compilés depuis les fichiers www/test*.html
    // dans ce cas on veut juste changer le publicPath
    // sinon c'est une arborescence à construire dans build/local
    if (!/^build\/local/.test(localPath)) {
      buildPath += `/${localPath}`
    }
    publicPath = localPath
    if (publicPath.substr(-1) !== '/') publicPath += '/'
  }
} else {
  buildPath = 'www/build/mtgLoad'
}

// set publicPath
if (isDevServer) {
  // un path bidon pour webpack-dev-server
  // on aura donc http://localhost:8080/devServer/mtgLoader.min.js qui pointera vers ce que construit
  // webpack-dev-server en ram, à la place de ce que webpack aurait mis dans le dossier de build
  // on précise le chemin complet (plutôt que `publicPath = 'devServer'`)
  // pour pouvoir appeler http://localhost:8080/devServer/mtgLoader.min.js
  // depuis un autre localhost:xxx et qu'il puisse charger ses dépendances
  // (pour tester mathgraph depuis une sesatheque locale par exemple)
  publicPath = `http://${host}:${port}/devServer/`
} else if (!publicPath && !isLocal) {
  // cette valeur par défaut si ça n'a pas été affecté avant
  publicPath = (isPreprod || isDev)
    ? 'https://dev.mathgraph32.org/js/mtgLoad/'
    : 'https://www.mathgraph32.org/js/mtgLoad/'
}

// la conf que l'on va exporter
const conf = {
  // cf https://webpack.js.org/concepts/#entry
  entry: {
    mtgLoad: './src/mtgLoad.js'
  },
  // cf https://webpack.js.org/concepts/#output
  output: {
    path: path.resolve(__dirname, buildPath),
    publicPath,
    filename: `mtgLoad${isDev || isElectron ? '' : '.min'}.js`,
    chunkFilename: '[chunkhash].js',
    // [name] est remplacé par le nom de la propriété de entry
    // cf https://webpack.js.org/configuration/output/#outputlibrary
    // exporte le module mis dans entry (attention, si y'en a plusieurs c'est le dernier)
    // en global dans cette variable
    library: '[name]',
    // comportement par défaut, mais pas plus mal en l'explicitant, pour le type d'export de la library, ici var => globale
    libraryTarget: 'var',
    // ça c'est pour charger les chunks en cross-domain
    crossOriginLoading: 'anonymous'
  },
  mode: isDev ? 'development' : 'production',
  module: {
    rules: [
      /* {
       // on ne passe par babel que notre code
       test: /src\/.*\.js/,
       loader: 'babel'
       },
       // et celui de binaryParser car il utilise les exports es6
       {
       // on ne passe par babel que notre code
       test: /outilsExternes\/binaryParser\/.*\.js/,
       loader: 'babel'
       }, /* */
      // bizarre, y'a du js hors de notre code qui utilise export, on laisse ça pour le moment
      {
        test: /\.js$/,
        exclude: /node_modules/,
        use: babelLoader
      },
      // les fichiers statiques
      { test: /\.(jpe?g|png|gif|otf|eot)(\?.*)?$/, loader: 'url-loader?limit=10000' },
      { test: /\.svg(\?\S*)?$/, loader: 'url-loader?mimetype=image/svg+xml&limit=10000' },
      { test: /\.ttf(\?\S*)?$/, loader: 'url-loader?mimetype=application/octet-stream&limit=10000' },
      { test: /\.woff2?(\?\S*)?$/, loader: 'url-loader?mimetype=application/font-woff&limit=10000' },
      // css, cf https://webpack.js.org/plugins/mini-css-extract-plugin/#advanced-configuration-example
      {
        test: /\.css$/,
        use: [
          {
            loader: MiniCssExtractPlugin.loader,
            options: {
              hmr: isDev
            }
          },
          'css-loader',
          {
            loader: 'postcss-loader',
            options: {
              plugins: () => [
                autoprefixer()
              ]
            }
          }
        ]
      }
    ]
  },
  plugins: [
    new MiniCssExtractPlugin({
      filename: '[name].css'
      // chunkFilename: isDev ? '[id].css' : '[id].[hash].css'
    })
  ],
  stats: {
    // indique l'heure de build, très utile en mode watch
    builtAt: true,
    // Nice colored output
    colors: true
  },
  // cf https://webpack.js.org/configuration/devtool/
  // Attention, faut aussi passer l'option au plugin terser (When using the terser-webpack-plugin
  // you must provide the sourceMap: true option to enable SourceMap support)
  // devtool: isDev ? 'cheap-module-eval-source-map' : 'source-map',
  devtool: hasSourceMap ? 'source-map' : undefined,
  // https://medium.com/webpack/webpack-4-mode-and-optimization-5423a6bc597a
  optimization: {
    // par défaut c'est false en dev et true en prod, décommenter la ligne suivante pour trouver l'origine d'un plantage de build en prod
    // noEmitOnErrors: false,

    minimizer: []
    // cf https://webpack.js.org/plugins/split-chunks-plugin/
  }
}

// on ajoute la minification js&css pour la prod
if (hasMinify) {
  const terserMinimizer = new TerserPlugin({
    // https://webpack.js.org/plugins/terser-webpack-plugin/
    // https://github.com/webpack-contrib/terser-webpack-plugin#terseroptions
    cache: true,
    parallel: true,
    // par défaut il laisse N fois les commentaires identiques (avec @licence) dans le fichier minifié
    // on ne veut pas extraire les commentaires dans un fichier .LICENSE séparé,
    // seulement n'en garder qu'un, d'où le output.comments ci-dessous
    extractComments: false,
    sourceMap: hasSourceMap, // Must be set to true if using source-maps
    // pour terserOptions cf https://github.com/terser-js/terser#minify-options
    terserOptions: {
      output: {
        // les seuls qu'on garde dans le fichier minifié
        // https://github.com/webpack-contrib/terser-webpack-plugin#preserve-comments
        // Attention, la chaîne passée à la regex démarre après "/*" (ou //)
        comments: /^\**!/
      },
      // https://github.com/webpack-contrib/terser-webpack-plugin#warningsfilter
      // avec 'verbose' c'est vraiment très verbeux, avec plein de warning qui n'en sont pas vraiment
      // (par ex il vire une variable utilisée une seule fois sur la ligne suivante,
      // mais c'est souvent fait exprès et bienvenu pour la lisibilité)
      // warnings: 'verbose'
      warnings: true
    }
  })
  conf.optimization.minimizer.push(terserMinimizer)
}

// pour la version locale, on peut vouloir tout dans un seul js
// (sinon ça marche avec devServer mais pas en ouvrant un html local, qui donnerait du
// file:///C:/…' from origin 'null' has been blocked by CORS policy…)
// @see https://webpack.js.org/plugins/limit-chunk-count-plugin/#maxchunks
// ATTENTION, ce truc casse le splitChunks css, et pour electron on se retrouve avec
// - plus de mtgLoad.min.css mais un mtgLoad.css
// - le loading css fait par le bootstrap webpack se fait plus => il faut préciser la css dans le index.html electron
if (isAllInOne) {
  conf.plugins.push(new webpack.optimize.LimitChunkCountPlugin({ maxChunks: 1 }))
} else if (hasMinify) {
  // on veut un seul css (https://webpack.js.org/plugins/mini-css-extract-plugin/#extracting-all-css-in-a-single-file)
  conf.optimization.splitChunks = {
    cacheGroups: {
      // pour mettre tous les styles requis par les ≠ js dans le même fichier css
      styles: {
        // ATTENTION, il ne faut pas indiquer ici le même nom qu'une entry, sinon ça plante avec
        // Cannot read property 'pop' of undefined
        // cf https://github.com/webpack-contrib/mini-css-extract-plugin/issues/257#issuecomment-421625285
        // et la doc https://webpack.js.org/plugins/split-chunks-plugin/#splitchunksname
        // dit que si l'on met le même qu'une entry existante ça vire l'entry
        name: '[contenthash].min',
        test: /\.css$/,
        chunks: 'all',
        enforce: true,
        // sans cette option, on a un seul fichier mais il s'appelle 0.mtgLoad.min.css, avec c'est bien mtgLoad.min.css
        reuseExistingChunk: true
      }
    }
  }
}

// on nettoie le dossier de build, sauf en prod|preprod
if (hasPurge) {
  conf.plugins.unshift(new CleanWebpackPlugin())
}

// pour devServer
if (isDevServer) {
  conf.devServer = {
    // ça c'est pour pouvoir charger depuis un http://truc.local qui ne serait pas dans les dns
    disableHostCheck: true,
    // pour pouvoir charger les chunks depuis un autre localhost:xxx
    headers: {
      'Access-Control-Allow-Origin': '*'
    },
    host,
    port
  }
}

module.exports = conf

// on affiche les infos de ce qu'on va compiler
console.log(`Mode           : ${conf.mode}`)
console.log(`Src            : ${conf.entry.mtgLoad}`)
console.log(`BuildDir       : ${conf.output.path}`)
console.log(`Clean BuildDir : ${(isDev || isLocal) ? 'yes' : 'no'}`)
console.log(`PublicPath     : ${publicPath}`)
console.log(`Electron       : ${isElectron ? 'yes' : 'no'}`)
console.log(`Local          : ${isLocal ? 'yes' : 'no'}`)
// y'a un truc louche, en console ça sort bien le mtgLoad.all.css défini plus haut mais ça compile un build/mtgLoad.css…
console.log(`Extract css    : ${(isDev || isAllInOne) ? 'no' : `yes (${conf.optimization.splitChunks.cacheGroups.styles.name}.css)`}`)
console.log(`Minify         : ${isDev ? 'no' : 'yes'}\n\n`)
// on ajoute l'url à tester suivant le cas
if (isDevServer) {
  console.log(`Pour le dev tester http://${host}:${port}/\n`)
  console.log('Vous pouvez passer une figure base64 à tester (XXX) en ajoutant #XXX ou #playerXXX')
}
// console.log('la config', JSON.stringify(conf))
// process.exit()
